﻿/****************************************
 *  VideoPlayer class (AS3) 
 *  by Andrei Neamtu
 *  2009 Vuzum Media, LLC. www.vuzum.com
 ***************************************/
package com.vuzum.mediaplayer
{
	// FLASH IMPORTS
	import flash.text.TextFieldAutoSize;
	import flash.utils.setInterval;
	import flash.display.MovieClip;
	import flash.display.StageDisplayState;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.utils.clearTimeout;
	import flash.utils.setTimeout;

	// OTHER IMPORTS
	//	import flash.ui.Mouse;
	import com.zorked.URLNavigator;
	import caurina.transitions.Tweener;
		

	// VUZUM IMPORTS
	import com.vuzum.mediaplayer.SoundManager;
	import com.vuzum.utils.*;


	/**
	 * CLASS
	 */
	public class MediaPlayer extends MovieClip 
	{
		// STATIC MEMBERS
		private static const VIDEOMODE : String = "video";
				
		/*
		 * COMPONENTS INSIDE
		 */
		public var mcLoader : MovieClip;
		//
		public var mcLogo : MovieClip;
		//
		public var mcImageHolder : MovieClip;
		//
		public var mcVideo : VideoManager;
		var soundManager : SoundManager;
		//
		public var mcList : ThumbsGallery;
		public var mcController : Controller;
		//		
		public var mcEmbed : Embed;
		public var mcMessage : MovieClip;
		//
		public var mcVideoButton : MovieClip;
		public var mcCenterPlayBt : MovieClip;
		//
		public var mcBack : MovieClip;

		/*
		 * COMPONENT'S VARS
		 */
		// player dimensions
		public var componentWidth : Number;
		public var componentHeight : Number;
		// path to files
		public var pathToFiles : String = "";
		// settings xml path
		public var xmlPath : String = "";
		// content xml path
		public var contentXMLPath : String = "";
		public var contentXmlObject : * = null;
		public var startWithItemNo : Number = 1;
		private var galleryHeight : Number;
		//
		// logo settings
		public var logoPath : String = "";
		public var logoText : String = "";
		public var logoURL : String;
		public var logoPosition : String = "TR"; // used to arange elemnts inside
		public var logoMargin : Number = 5; // used to arange elemnts inside		
		// to do on hiding the controller
		public var hideListener : Function;
		public var downloadScript : String = "download_file.php";	

		
		/*
		 * INTERNAL MEMBERS
		 */
		// MODE
		public var mode : String = VIDEOMODE;
		// current VIDEO PATH and STREAMER
		public var videoFilePath : String = "";
		public var streamer : String;
		// LQ video path
		public var duration : Number;
		public var lqVideoPath : String = "";
		public var streamerLQ : String;
		// HD video path
		public var hdVideoPath : String = "";
		public var streamerHD : String;
		public var startWithHD : Boolean = false;

		// current PREVIEW PATH
		public var previewFilePath : String = "";
		public var afterImageIsShown : Function = null;	// to do after the preview is shown	

		// player embed object
		public var embedObject : Object;
		
		// handlers vars
		private var currPressedItem : MovieClip; // used for press and release event		


		/*
		 * SETTINGS COMMING FROM OUTSIDE
		 */
		// playing settings
		public var autoStartPlayer : Boolean = true; // on first load
		public var autoStart : Boolean = true; // usually
		public var autoPlay : Boolean = true; // usually
		//
		public var bufferTime : Number = 2;
		//
		public var volumeXMLObject : * = null;
		public var startVolume : Number = 50;
		public var fadeSoundTime : Number = 0;
	
		// controller general settings
		public var controllerUnder : Boolean = false;
		public var controllerHeight : Number = 30;
		public var controllerMargin : Number = 0;
		// auto hide settings
		public var controllerAutohideTime : Number = -1; // means it will not autohide
		public var controllerShowTime : Number = 0.7; // for tween
		private var autoHideTimeOutId : uint;
		
		// controls general settings
		public var controlsDimension : Number = 35;		
		public var controlsDistanceBetween : Number = 0;
		public var controlsSignFillColor : Number;
		public var controlsSignFillOverColor : Number;
		// center play bt settings
		public var disableCenterPlayBt : Boolean = true;
		public var centerPlayBtSize : Number;
		public var normalCenterBtAlpha : Number = 0.5;
		public var overCenterBtAlpha : Number = 1;
		// stop bt settings
		public var disableStopBt : Boolean = true;
		// playing bar settings
		public var playbarOffset : Number = 5;		
		// voliume ct settings
		public var disableVolumeCt : Boolean = false;
		public var disableVolumeBt : Boolean = false;
		public var verticalVolume : Boolean = false;		
		public var volumeBarWidth : Number;
		public var volumeBarHeight : Number;
		public var volumeScrubWidth : Number;
		public var volumeScrubHeight : Number;				
		// full bt settings
		public var disableFullBt : Boolean = false;
		public var disableThumbsBt : Boolean = false;
		// embed button settings
		public var disableEmbedBt : Boolean = false;
		public var useEmbedBtAsDownloadBt : Boolean = false;
		public var embedBtText : String = "";
		public var embedBtTextSize : Number = 12;
		public var embedWindowXMLObj : *;
		// hd button settings
		public var disableHdBt : Boolean = false;
		public var hdToggleTextSize : Number = 12;
		public var hdToggleSDText : String = "HD";
		public var hdToggleHDText : String = "SD";
		// timer settings
		public var disableTimer : Boolean = false;
		public var timerTextSize : Number;
		// skip controls
		public var skipControls : Boolean = false; // for a playlist
		//
		public var disableRandomBt : Boolean = false;
		
		
		/*
		 * AUX VARS
		 */
		//position of the player in normal state
		private var initPlayerXPos : Number;
		private var initPlayerYPos : Number;
		//position of the player in full state
		private var fullPlayerXPos : Number;
		private var fullPlayerYPos : Number;
		//
		public var isFullScreen : Boolean = false;
		//		
		public var firstPlay : Boolean = true;
		public var showPlayFlag : Boolean = false; // toogle to show center play button when using player as image viewer
		
		// vars used inside and in all classes
		private var previewLoaded : Boolean = false;
		//
		private var progressIntervalId : uint;
		private var downloadIntervalId : uint;
		private var timeToStayId : Number;

		/*
		 * CONSTRUCTOR
		 */
		function MediaPlayer()
		{
			//trace("VIDEOPLAYER CONSTRUCTOR");

			// init PROTOTYPES			
			new Prototypes();
						
			// init SOUND MANAGER and hide everything
			soundManager = new SoundManager();
			
	    	// add listener to know when the video starts to download
	    	toggleListeners();			
			
			// INIT ELMENTS INSIDE
			mcMessage.mcText.txt.embedFonts = true;
			mcMessage.mcText.txt.autoSize = TextFieldAutoSize.LEFT;
			mcMessage.visible = false;
			//
			mcList.visible = false;
			//
			mcEmbed.visible = false;
			//
			mcCenterPlayBt.visible = false;
			mcCenterPlayBt.mcOver.visible = false;
			//
			mcLogo.visible = false;
			//
			show(false, true);
		}
		
		// add listener to know when the video is loaded/buffered/ended
		private function toggleListeners(flag : Boolean = true) : void
		{
	    	
//	    	trace("ADD LISTENERS");	    	
	    	if(flag)
	    	{
	    		if(isPlayingAudio)
	    		{
					soundManager.addEventListener("MediaBufferFull", mediaBufferFullListener);			
					soundManager.addEventListener("MediaLoaded", mediaLoadedListener);			
					soundManager.addEventListener("MediaPlayingStart", mediaPlayingStartListener);
					soundManager.addEventListener("MediaPlayingPause", mediaPlayingPauseListener);
					soundManager.addEventListener("MediaPlayingEnd", mediaPlayingEndListener);
					soundManager.addEventListener("MediaPlayingStop", mediaPlayingStopListener);
	    		}
				else
				{
					mcVideo.addEventListener("MediaBufferFull", mediaBufferFullListener);			
					mcVideo.addEventListener("MediaLoaded", mediaLoadedListener);			
					mcVideo.addEventListener("MediaPlayingStart", mediaPlayingStartListener);
					mcVideo.addEventListener("MediaPlayingPause", mediaPlayingPauseListener);
					mcVideo.addEventListener("MediaPlayingStop", mediaPlayingStopListener);
				}	
	    	}
	    	else
	    	{
				soundManager.removeEventListener("MediaBufferFull", mediaBufferFullListener);			
				soundManager.removeEventListener("MediaLoaded", mediaLoadedListener);			
				soundManager.removeEventListener("MediaPlayingStart", mediaPlayingStartListener);
				soundManager.removeEventListener("MediaPlayingPause", mediaPlayingPauseListener);
				soundManager.removeEventListener("MediaPlayingEnd", mediaPlayingEndListener);
				soundManager.removeEventListener("MediaPlayingStop", mediaPlayingStopListener);
				//
				mcVideo.removeEventListener("MediaBufferFull", mediaBufferFullListener);			
				mcVideo.removeEventListener("MediaLoaded", mediaLoadedListener);			
				mcVideo.removeEventListener("MediaPlayingStart", mediaPlayingStartListener);
				mcVideo.removeEventListener("MediaPlayingPause", mediaPlayingPauseListener);
				mcVideo.removeEventListener("MediaPlayingEnd", mediaPlayingEndListener);
				mcVideo.removeEventListener("MediaPlayingStop", mediaPlayingStopListener);
	    	}
		}

		/**
		 * INIT THE COMPONENT with the settings from a xml file and the content from an xml file
		 */
		public function init(pComponentWidth : Number = NaN, pComponentHeight : Number = NaN, pPathToFiles : String = "", 
							 
							 pmode : String = "", 
							 
							 pXmlPath : String = "", pContentXmlPath : String = "", pstartWithItemNo : Number = 1, 
							 
							 pVideoPath : String = "", pStreamer : String = "", 
							 pVideoHDPath : String = "", pStreamerHD : String = "", 
							 pPreviewPath : String = "",
							 
							 pDownloadScript : String = "",
							 
							 plogoPath : String = "", plogoURL : String = "", plogoPosition : String = "", plogoMargin : Number = 5) : void
		{
			// READ THE PARAMS
			// dimensions
			componentWidth = pComponentWidth;
			componentHeight = pComponentHeight;
			
			// path to files
			pathToFiles = Prototypes.readPathTo(pPathToFiles);
			
			
			// EMBED OBJECT
			embedObject = new Object();
			// dimensions
			embedObject.componentWidth = componentWidth;
			embedObject.componentHeight = componentHeight;
			// path to files
			embedObject.pathToFiles = pathToFiles;
			// video single mode paths
			embedObject.videoPath = Prototypes.readStringVar(pVideoPath);
			embedObject.streamer = Prototypes.readStringVar(pStreamer, null);
			embedObject.videoHDPath = Prototypes.readStringVar(pVideoHDPath);
			embedObject.streamerHD = Prototypes.readStringVar(pStreamerHD, null);
			embedObject.previewPath = Prototypes.readStringVar(pPreviewPath);			
			// xml paths
			embedObject.xmlPath = Prototypes.readStringVar(pXmlPath);
			embedObject.contentXMLPath = (embedObject.videoPath != "" ? "" : Prototypes.readStringVar(pContentXmlPath));
			embedObject.startWithItemNo = Prototypes.readNumberVar(pstartWithItemNo, 1);
			embedObject.downloadScript = Prototypes.readStringVar(pDownloadScript);
			// logo paths
			embedObject.logoPath = Prototypes.readStringVar(plogoPath);
			embedObject.logoURL = Prototypes.readStringVar(plogoURL);
			embedObject.logoPosition = Prototypes.readStringVar(plogoPosition);
			embedObject.logoMargin = Prototypes.readNumberVar(plogoMargin);
			
			
			// SINGLE VIDEO MODE assets paths
			mode = Prototypes.readStringVar(pmode, VIDEOMODE);
			lqVideoPath = Prototypes.readAssetPath(pVideoPath, pathToFiles); //trace("lqVideoPath:", lqVideoPath);
			streamer = embedObject.streamer;
			hdVideoPath = Prototypes.readAssetPath(pVideoHDPath, pathToFiles);
			streamerHD = embedObject.streamerHD;
			previewFilePath = Prototypes.readAssetPath(pPreviewPath, pathToFiles);
			
			// xml paths
			xmlPath = Prototypes.readAssetPath(embedObject.xmlPath, pathToFiles);
			contentXMLPath = (lqVideoPath != "" ? "" : Prototypes.readAssetPath(embedObject.contentXMLPath, pathToFiles));
			startWithItemNo = embedObject.startWithItemNo;
			contentXmlObject = null;

			// download script path
			downloadScript = Prototypes.readAssetPath(Prototypes.readStringVar(pDownloadScript, downloadScript), pathToFiles);
			
			// logo paths
			logoPath = Prototypes.readAssetPath(embedObject.logoPath, pathToFiles);
			logoURL = embedObject.logoURL;
			logoPosition = embedObject.logoPosition;
			logoMargin = embedObject.logoMargin;			

			
			
			// HIDE EVERYTHING and show loader
			show(false, true);
			// repos and show the loader
			reposLoader();
			
			
			// LOAD XML and then start the component
			if(xmlPath != "") Prototypes.loadXML(xmlPath, xmlLoadedHandler, mcLoader);
			else xmlLoadedHandler(null);
		}

		/**
		 * INIT THE COMPONENT with the settings from a xml object
		 */
		public function initWithXMLObject(pComponentWidth : Number = NaN, pComponentHeight : Number = NaN, pPathToFiles : String = "", pmode : String = "", pXmlObject : * = null) : void
		{
			// READ THE PARAMS
			componentWidth = pComponentWidth;
			componentHeight = pComponentHeight;
			
			// path to files
			pathToFiles = pPathToFiles;
			var url : String = stage.loaderInfo.url;
			var pozLastSlash : uint = url.lastIndexOf("/");
			//var filename : String = url.substring(pozLastSlash, url.length);
			//trace(filename);
			
			// mode
			mode = Prototypes.readStringVar(pmode, VIDEOMODE);
			
			// COMPLETE EMBEDOBJECT
			embedObject = new Object();
			// path to files
			if(pathToFiles.indexOf("http://") != -1) embedObject.pathToFiles = pathToFiles;
			else embedObject.pathToFiles = url.substring(0, pozLastSlash + 1) + pathToFiles;
			embedObject.componentWidth = componentWidth;
			embedObject.componentHeight = componentHeight;
			// xml path
			embedObject.xmlPath = xmlPath;
			if((embedObject.xmlPath as String).indexOf("http://") == -1)
			{
				embedObject.xmlPath = embedObject.pathToFiles + (embedObject.xmlPath as String).substr(pathToFiles.length, (embedObject.xmlPath as String).length);
			}
			// start with tab and item no
			embedObject.videoToStartWith = "1@1";
			embedObject.videoToStartWithOnShare = "1@1";
			// content xml path
			//embedObject.contentXmlPath = pContentXmlPath;				


			// HIDE EVERYTHING and show loader
			show(false, true);
			// repos and show the loader
			reposLoader();
			
			// LOAD XML and then start the component
			xmlLoadedHandler(pXmlObject);
		}		

		/*
		 * TO DO AFTER THE settings XML has loaded
		 */
		public function xmlLoadedHandler(xmlObject : *) : void 
		{	
			var tempVar : *;
			
			// READ THE SETTINGS
			if(xmlObject)
			{
				// read the dimensions
				if(isNaN(componentWidth)) componentWidth = Prototypes.readNumberVar(xmlObject.@width, stage.stageWidth);
				if(isNaN(componentHeight)) componentHeight = Prototypes.readNumberVar(xmlObject.@height, stage.stageHeight);
				//trace(componentWidth, componentHeight);    	
				//embedObject.componentWidth = componentWidth;
				//embedObject.componentHeight = componentHeight;				
		    		
		    				
				// read the SETTINGS NODE
				var settingsNode : * = xmlObject; //trace(typeof(settingsNode));	//trace(settingsNode.children());
				var colorsNode : * = xmlObject;
				
				// PLAYING SETTINGS
				// autostart
				autoStartPlayer = Prototypes.readBooleanVar(settingsNode.autoStartPlayer); //trace(autoStart);
				autoStart = Prototypes.readBooleanVar(settingsNode.autoPlay); //trace(autoStart);
				startWithHD = Prototypes.readBooleanVar(settingsNode.startWithHD, false); //trace(startWithHD);
				// buffer time
				bufferTime = Prototypes.readNumberVar(settingsNode.bufferTime, bufferTime);
				// init volume
				startVolume = Prototypes.readNumberVar(settingsNode.initVolumePercent, startVolume) / 100;
				fadeSoundTime = Prototypes.readNumberVar(settingsNode.initVolumePercent.@fadeSoundTime, 0);
				//
				soundManager.setFadeTime = fadeSoundTime;
				soundManager.setVolumeInstant(startVolume);
				//
				mcVideo.setFadeTime = fadeSoundTime;
				mcVideo.setVolumeInstant(startVolume);
				

				// BACKGROUND settings
				mcBack.alpha = Prototypes.readNumberVar(settingsNode.backgroundColor.@opacity, 100) / 100;
				mcBack.setColor(Prototypes.readNumberVar(settingsNode.backgroundColor));					
				
				
				// CONTROLLER settings
				// background colors
				mcController.mcBack.alpha = Prototypes.readNumberVar(settingsNode.controllerBackColor.@opacity, 100) / 100;
				tempVar = Prototypes.readNumberVar(settingsNode.controllerBackColor.@strokeColor);
				if(tempVar >= 0) mcController.mcBack.mcStroke.setColor(tempVar);
				else mcController.mcBack.mcStroke.visible = false;
				mcController.mcBack.mcFill.setColor(Prototypes.readNumberVar(settingsNode.controllerBackColor.@fillColor));
				//
				tempVar = Prototypes.readNumberVar(settingsNode.controllerBackColor.@fillColor);
				mcController.mcBack.mcFill.setColor(tempVar);
//				mcController.mcVolume.mcVolumeBar.mcBack.setColor(tempVar);				
				// height
				controllerHeight = Prototypes.readNumberVar(settingsNode.controllerHeight, controllerHeight);
				controllerMargin = Prototypes.readNumberVar(settingsNode.controllerMargin, controllerMargin);
				controlsDistanceBetween = Prototypes.readNumberVar(settingsNode.controlsDistanceBetween, controlsDistanceBetween);
				controlsDimension = Prototypes.readNumberVar(settingsNode.controlsDimension, controlsDimension);
				// under video flag
				controllerUnder = Prototypes.readBooleanVar(settingsNode.controllerUnder); //trace(controllerUnder);		
				// autoHide Time
				controllerAutohideTime = Prototypes.readNumberVar(settingsNode.controllerAutohideTime, -1);
				
				// CENTER PLAY BT settings
				disableCenterPlayBt = Prototypes.readBooleanVar(settingsNode.centerPlayBt.@disable); //trace(controllerUnder);
				centerPlayBtSize = Prototypes.readNumberVar(settingsNode.centerPlayBt.@size, mcCenterPlayBt.width); //trace(controllerUnder);
				// sign fill / stroke color
				tempVar = Prototypes.readNumberVar(settingsNode.centerPlayBt.signStrokeColor);
				if(tempVar >= 0) mcCenterPlayBt.mcSign.mcStroke.setColor(tempVar);
				else mcCenterPlayBt.mcSign.mcStroke.visible = false;
				mcCenterPlayBt.mcSign.mcFill.setColor(Prototypes.readNumberVar(settingsNode.centerPlayBt.signFillColor));
				// back fill / stroke color
				tempVar = Prototypes.readNumberVar(settingsNode.centerPlayBt.backStrokeColor);
				if(tempVar >= 0) mcCenterPlayBt.mcBack.mcStroke.setColor(tempVar);
				else mcCenterPlayBt.mcBack.mcStroke.visible = false;
				tempVar = Prototypes.readNumberVar(settingsNode.centerPlayBt.backFillColor);
				if(tempVar >= 0) mcCenterPlayBt.mcBack.mcFill.setColor(tempVar);
				else mcCenterPlayBt.mcBack.mcFill.alpha = 0;				
				// normal / over alpha
				normalCenterBtAlpha = Prototypes.readNumberVar(settingsNode.centerPlayBt.normalOpacity, 100) / 100;
				overCenterBtAlpha = Prototypes.readNumberVar(settingsNode.centerPlayBt.overOpacity, 100) / 100;
				
				
				
				// CONTROLS SIGN colors
				controlsSignFillColor = Prototypes.readNumberVar(colorsNode.controlsSignFillColor);
				controlsSignFillOverColor = Prototypes.readNumberVar(colorsNode.controlsSignFillColor.@over);
				mcController.mcVolumeCt.btSignOverColor = controlsSignFillOverColor;
				if(!isNaN(controlsSignFillColor))
				{
					mcController.mcStopBt.mcSign.mcFill.setColor(controlsSignFillColor);
//					
					mcController.mcPlayBt.mcSign.mcFill.setColor(controlsSignFillColor);
					mcController.mcPauseBt.mcSign.mcFill.setColor(controlsSignFillColor);
//					
					mcController.mcVolumeCt.btSignNormalColor = controlsSignFillColor;
//
					mcController.mcEmbedBt.mcSign.setColor(controlsSignFillColor);
					//
					mcController.mcRepeatBt.mcSign.setColor(controlsSignFillColor);
					mcController.mcShuffleBt.mcSign.setColor(controlsSignFillColor);
					//
					mcController.mcRewindBt.mcSign.mcFill.setColor(controlsSignFillColor);
					mcController.mcForwardBt.mcSign.mcFill.setColor(controlsSignFillColor);
					//
					mcController.mcThumbsBt.mcSign.setColor(controlsSignFillColor);
					mcController.mcHdtoggleBt.mcSign.setColor(controlsSignFillColor);
					//
					mcController.mcFullBt.mcSign.setColor(controlsSignFillColor);
				}
				// stroke color
				tempVar = Prototypes.readNumberVar(colorsNode.controlsSignStrokeColor);
				if(tempVar >= 0)
				{				
					mcController.mcStopBt.mcSign.mcStroke.setColor(tempVar);
					//
					mcController.mcPlayBt.mcSign.mcStroke.setColor(tempVar);
					mcController.mcPauseBt.mcSign.mcStroke.setColor(tempVar);
					//
					mcController.mcRewindBt.mcSign.mcStroke.setColor(tempVar);
					mcController.mcForwardBt.mcSign.mcStroke.setColor(tempVar);										
					//
//					mcController.mcFullBt.mcSign.mcSmall.setColor(tempVar);
//					mcController.mcFullBt.mcSign.mcBig.setColor(tempVar);
				}
				else
				{
					mcController.mcStopBt.mcSign.mcStroke.visible = false;
					//
					mcController.mcPlayBt.mcSign.mcStroke.visible = false;
					mcController.mcPauseBt.mcSign.mcStroke.visible = false;
					//
					mcController.mcRewindBt.mcSign.mcStroke.visible = false;
					mcController.mcForwardBt.mcSign.mcStroke.visible = false;
					//
//					mcController.mcFullBt.mcSign.mcSmall.visible = false;
//					mcController.mcFullBt.mcSign.mcBig.visible = false;							
				}
				
				// CONTROS BACK colors
				// stroke color
				tempVar = Prototypes.readNumberVar(colorsNode.controlsBackStrokeColor);
				if(tempVar >= 0)
				{					
					mcController.mcStopBt.mcBack.mcStroke.setColor(tempVar);
					mcController.mcPauseBt.mcBack.mcStroke.setColor(tempVar);
					mcController.mcPlayBt.mcBack.mcStroke.setColor(tempVar);
					//
//					mcController.mcVolume.mcVolumeBt.mcBack.mcStroke.setColor(tempVar);
					mcController.mcVolumeCt.btStrokeColor = tempVar;
//					//
					mcController.mcThumbsBt.mcBack.mcStroke.setColor(tempVar);
					mcController.mcHdtoggleBt.mcBack.mcStroke.setColor(tempVar);
					//
					mcController.mcEmbedBt.mcBack.mcStroke.setColor(tempVar);
					//
					mcController.mcShuffleBt.mcBack.mcStroke.setColor(tempVar);
					mcController.mcRepeatBt.mcBack.mcStroke.setColor(tempVar);
					//
					mcController.mcRewindBt.mcBack.mcStroke.setColor(tempVar);
					mcController.mcForwardBt.mcBack.mcStroke.setColor(tempVar);										
//					//
					mcController.mcFullBt.mcBack.mcStroke.setColor(tempVar);
				}
				else 
				{
					mcController.mcStopBt.mcBack.mcStroke.visible = false;
					mcController.mcPauseBt.mcBack.mcStroke.visible = false;
					mcController.mcPlayBt.mcBack.mcStroke.visible = false;
					//
//					mcController.mcVolume.mcVolumeBt.mcBack.mcStroke.visible = false;
//					//
					mcController.mcThumbsBt.mcBack.mcStroke.visible = false;
					mcController.mcHdtoggleBt.mcBack.mcStroke.visible = false;
					//
					mcController.mcEmbedBt.mcBack.mcStroke.visible = false;
					//
					mcController.mcShuffleBt.mcBack.mcStroke.visible = false;
					mcController.mcRepeatBt.mcBack.mcStroke.visible = false;
					//
					mcController.mcRewindBt.mcBack.mcStroke.visible = false;
					mcController.mcForwardBt.mcBack.mcStroke.visible = false;
//					//
					mcController.mcFullBt.mcBack.mcStroke.visible = false;					
				}
				// fill color
				tempVar = Prototypes.readNumberVar(colorsNode.controlsBackFillColor);
				if(tempVar >= 0)
				{				
					mcController.mcStopBt.mcBack.mcFill.setColor(Number(tempVar));
					mcController.mcPauseBt.mcBack.mcFill.setColor(Number(tempVar));
					mcController.mcPlayBt.mcBack.mcFill.setColor(Number(tempVar));
					//
					mcController.mcVolumeCt.btFillColor = tempVar;
//					//
					mcController.mcThumbsBt.mcBack.mcFill.setColor(Number(tempVar));
					mcController.mcHdtoggleBt.mcBack.mcFill.setColor(Number(tempVar));
					//
					mcController.mcEmbedBt.mcBack.mcFill.setColor(Number(tempVar));
					//
					mcController.mcShuffleBt.mcBack.mcFill.setColor(tempVar);
					mcController.mcRepeatBt.mcBack.mcFill.setColor(tempVar);
					//
					mcController.mcRewindBt.mcBack.mcFill.setColor(tempVar);
					mcController.mcForwardBt.mcBack.mcFill.setColor(tempVar);									
//					//
					mcController.mcFullBt.mcBack.mcFill.setColor(Number(tempVar));
				}
				else
				{
//					mcController.mcStopBt.mcBack.mcFill.alpha = 0;
//					mcController.mcPauseBt.mcBack.mcFill.alpha = 0;
//					mcController.mcPlayBt.mcBack.mcFill.alpha = 0;
					//
//					mcController.mcVolume.mcVolumeBt.mcBack.mcFill.alpha = 0;
//					//
//					mcController.mcThumbsBt.mcBack.mcFill.alpha = 0;
//					mcController.mcHdtoggleBt.mcBack.mcFill.alpha = 0;
//					mcController.mcEmbedBt.mcBack.mcFill.alpha = 0;
////					//
//					mcController.mcFullBt.mcBack.mcFill.alpha = 0;									
				}
						
				
				// STOP Bt
				disableStopBt = Prototypes.readBooleanVar(settingsNode.disableStopBt, true); //trace(disableStopBt);
				
				// HD TOGGLE button settings
				disableHdBt = Prototypes.readBooleanVar(settingsNode.hdToggleBt.@disable, true);
				hdToggleTextSize = Prototypes.readNumberVar(settingsNode.hdToggleBt.@textSize, hdToggleTextSize);
				hdToggleSDText = Prototypes.readStringVar(settingsNode.hdToggleBt.sdText, hdToggleSDText);
				hdToggleHDText = Prototypes.readStringVar(settingsNode.hdToggleBt.hdText, hdToggleHDText);
				
				// TIMER settings
				disableTimer = Prototypes.readBooleanVar(settingsNode.disableTimer);
				timerTextSize = Prototypes.readNumberVar(settingsNode.timer.@textSize, 8); //trace(timerTextSize);
				//
				tempVar = Prototypes.readNumberVar(settingsNode.timer.backStrokeColor);
				if(tempVar >= 0) mcController.mcTimer.mcBack.mcStroke.setColor(Number(tempVar));
				else mcController.mcTimer.mcBack.mcStroke.visible = false;
				tempVar = Prototypes.readNumberVar(settingsNode.timer.backFillColor);
				if(tempVar >= 0) mcController.mcTimer.mcBack.mcFill.setColor(Number(tempVar));
				else mcController.mcTimer.mcBack.mcFill.visible = false;				
				//
				mcController.mcTimer.mcSign.tPassedTime.setColor(Prototypes.readNumberVar(settingsNode.timer.passedTextColor, controlsSignFillColor));
				mcController.mcTimer.mcSign.tTotalTime.setColor(Prototypes.readNumberVar(settingsNode.timer.totalTextColor, controlsSignFillColor));
				mcController.mcTimer.mcSign.tSeparator.setColor(Prototypes.readNumberVar(settingsNode.timer.separatorColor, controlsSignFillColor));				
				
				// EMBED BUTTON settings
				disableEmbedBt = Prototypes.readBooleanVar(settingsNode.embedBt.@disable);
				useEmbedBtAsDownloadBt = Prototypes.readBooleanVar(settingsNode.embedBt.@useAsDownloadBt, useEmbedBtAsDownloadBt);
				embedBtText = Prototypes.readStringVar(settingsNode.embedBt);//, "SHARE");				
				embedBtTextSize = Prototypes.readNumberVar(settingsNode.embedBt.@textSize, embedBtTextSize);				
				// EMBED WINDOW settings
				embedWindowXMLObj = settingsNode.embedWindow;

				// PLAYING BAR settings
				// bars settings
				mcController.mcPlayBar.mcBackBar.height = Prototypes.readNumberVar(colorsNode.playBar.barsHeight);
				mcController.mcPlayBar.mcBackBar.x = Prototypes.readNumberVar(colorsNode.playBar.barsHeight.@marginFromBack, 0);
				//
				mcController.mcPlayBar.mcPassedBar.setColor(Prototypes.readNumberVar(colorsNode.playBar.progressBarColor));
				mcController.mcPlayBar.mcDownloadBar.setColor(Prototypes.readNumberVar(colorsNode.playBar.downloadBarColor));
				mcController.mcPlayBar.mcBackBar.setColor(Prototypes.readNumberVar(colorsNode.playBar.backBarColor));
				// back settings
				tempVar = Prototypes.readNumberVar(colorsNode.playBar.backColor);
				if(tempVar >=0) mcController.mcPlayBar.mcBack.setColor(tempVar);
				else mcController.mcPlayBar.mcBack.visible = false;
				mcController.mcPlayBar.mcBack.alpha = Prototypes.readNumberVar(colorsNode.playBar.backColor.@opacity, 0) / 100;				
				mcController.mcPlayBar.mcBack.height = 2 * mcController.mcPlayBar.mcBackBar.x + mcController.mcPlayBar.mcBackBar.height;
				// scrub settings
				mcController.mcPlayBar.mcScrub.width = Prototypes.readNumberVar(colorsNode.playBar.scrubWidth);
				mcController.mcPlayBar.mcScrub.height = Prototypes.readNumberVar(colorsNode.playBar.scrubHeight);
				mcController.mcPlayBar.mcScrub.setColor(Prototypes.readNumberVar(colorsNode.playBar.scrubColor));

				
				
				// VOLUME CONTROL settings
				volumeXMLObject = settingsNode.volumeCt;
				disableVolumeCt = Prototypes.readBooleanVar(settingsNode.volumeCt.@disable);
				disableVolumeBt = Prototypes.readBooleanVar(settingsNode.volumeCt.@disableVolumeBt);
				verticalVolume = Prototypes.readBooleanVar(settingsNode.volumeCt.@vertical);
				// bars settings
				volumeBarWidth = Prototypes.readNumberVar(settingsNode.volumeCt.barsWidth);
				volumeBarHeight = Prototypes.readNumberVar(settingsNode.volumeCt.barsHeight);
				//
//				tempVar = Prototypes.readNumberVar(colorsNode.volumeCt.levelBarColor);
//				mcController.mcVolume.mcHVolumeBar.mcLevelBar.setColor(tempVar);
//				mcController.mcVolume.mcVolumeBar.mcSeekBar.setColor(tempVar);		
//				tempVar = Prototypes.readNumberVar(colorsNode.volumeCt.backBarColor);
//				mcController.mcVolume.mcHVolumeBar.mcBackBar.setColor(tempVar);		
//				mcController.mcVolume.mcVolumeBar.mcBackBar.setColor(tempVar);		
//				// scrub settings
//				mcController.mcVolume.mcHVolumeBar.mcScrub.width = volumeScrubWidth = Prototypes.readNumberVar(settingsNode.volumeCt.scrubWidth);
//				mcController.mcVolume.mcHVolumeBar.mcScrub.height = volumeScrubHeight = Prototypes.readNumberVar(settingsNode.volumeCt.scrubHeight);
//				mcController.mcVolume.mcHVolumeBar.mcScrub.setColor(settingsNode.volumeCt.scrubColor);			
//				// BACK settings
//				tempVar = Prototypes.readNumberVar(colorsNode.volumeCt.backColor);
//				if(tempVar >= 0) 
//				{
//					mcController.mcVolume.mcVolumeBar.mcBack.setColor(tempVar);				
//					mcController.mcVolume.mcBack.setColor(tempVar);				
//				}
//				else 
//				{
//					mcController.mcVolume.mcVolumeBar.mcBack.visible = false;
//					mcController.mcVolume.mcBack.visible = false;
//				}				
				
				// RANDOM BUTTON
				disableRandomBt = !Prototypes.readBooleanVar(settingsNode.shuffleBt, false);
				
				// FULL SCREEN BUTTON settings
				disableFullBt = Prototypes.readBooleanVar(settingsNode.disableFullBt, true); //trace(disableFullBt);					
				
				// THUMBS button settings
				disableThumbsBt = Prototypes.readBooleanVar(settingsNode.disableThumbsBt); //trace(disableThumbsBt);					
			}
			
			// LOAD CONTENT XML and then start the component after
			if(contentXMLPath != "") Prototypes.loadXML(contentXMLPath, contentXmlLoadedHandler, mcLoader);
			
			// START PLAYER
			else startPlayer();					
		}
		
		/*
		 * TO DO AFTER THE settings XML has loaded
		 */
		public function contentXmlLoadedHandler(xmlObject : *) : void 
		{	
			// read the content xml
			contentXmlObject = xmlObject;
			galleryHeight = Prototypes.readNumberVar(contentXmlObject.@height, mcList.mcBack.height);
			
			// START PLAYER
			startPlayer();			
		}		

		/*
		 * START the component after the settings have been read
		 */
		public function startPlayer() : void
		{	
			// RETAIN THE INITIAL POSITION OF THE PLAYER
			initPlayerXPos = fullPlayerXPos = this.x; 
			initPlayerYPos = fullPlayerYPos = this.y;  
//	    	trace(initPlayerXPos, initPlayerYPos);

			// CHANGE MODE
			if(!isVideoMode) 
			{
				controllerAutohideTime = -1;
				controllerUnder = false;
				controllerHeight = componentHeight;
				disableCenterPlayBt = true;
			}
			
			// LOADER
			mcLoader.visible = false;
			
			// BACKGROUND
			mcBack.x = mcBack.y = 0;

			// MESSAGE BOX
			mcMessage.visible = false;
			
			// CENTER PLAY BUTTON
			mcCenterPlayBt.width = mcCenterPlayBt.height = centerPlayBtSize;
			
			// VIDEO BUTTON
			//mcVideoButton.buttonMode = true;
			
			// CENTER BUTTON
			//			mcCenterPlayBt.addEvents(this);
			
			// IMAGE HOLDER
//			mcImageHolder
			
			// VIDEO HOLDER
			firstPlay = true;
			mcVideo.setBufferTime = bufferTime;
			mcVideo.init(componentWidth, componentHeight, this);
			
			// SOUND MANAGER
			soundManager.setBufferTime = bufferTime;

			// CONTROLLER
			disableThumbsBt = !hasContent;
			mcController.init(componentWidth - 2 * controllerMargin, controllerHeight, this);
			toggleAutohide();
			
		
			// MEDIA LIST
			mcList.initWithXMLObject(NaN, NaN, pathToFiles, (hasContent ? contentXmlObject : null), listItemReleaseFunction);
			mcList.visible = hasContent;
			if(hasContent) mcController.releaseFunction(mcController.mcThumbsBt);
			
			
			// EMBED WINDOW
			if(!disableEmbedBt)
			{
				mcEmbed.x = 0; //mcBack.x + mcBack.width - mcShare.width;
				mcEmbed.y = mcBack.y + mcBack.height;
				mcEmbed.init(mcBack.width, mcBack.height, this, embedWindowXMLObj);
			}
			else embedObject = null;
			mcEmbed.visible = false;
			
			// RESIZE HANDLER to treat full/normal screen state
			stage.addEventListener(Event.RESIZE, fullScreenHandler);
			resize(componentWidth, componentHeight);
			
			// LOGO
			mcLogo.alpha = 0;
			mcLogo.loaded = false;
			mcLogo.mcHolder.scaleX = mcLogo.mcHolder.scaleY = 0;
			mcLogo.txt.scaleX = mcLogo.txt.scaleY = 0;
			if(logoPath != "" || logoText != "")// && logoObject != undefined)
			{
				// show logo
				mcLogo.visible = false;
				
				// load the image
				if(logoPath != "") 
				{
					mcLogo.mcHolder.scaleX = mcLogo.mcHolder.scaleY = 1;
					
					// load image
					mcLogo.mcHolder.loadContent(logoPath, logoLoaded);
				}

				// write the text and resize the background if there is one
				else
				if(logoText != "") 
				{
					mcLogo.txt.scaleX = mcLogo.txt.scaleY = 1;
					
					// write the text
					mcLogo.txt.embedFonts = true;
					mcLogo.txt.autoSize = TextFieldAutoSize.LEFT;
					mcLogo.txt.text = logoText;
					
					// START SHOWING the image holder
					repositionLogo();
					mcLogo.showObject();					
				}
			
				
				// add link event if there is one
				if(logoURL != "")
				{
					mcLogo.buttonMode = true;
					mcLogo.addEventListener(MouseEvent.MOUSE_DOWN, pressHandler);
					mcLogo.addEventListener(MouseEvent.MOUSE_UP, releaseHandler);
				}
			}
			else
			{
				mcLogo.visible = false;
			}			
			
			
			// SHOW EVERYTHING
			show(true);			
			
			// START THE VIDEOS IS NOT EMPTY
			if(lqVideoPath != "") 
			{
				trace("load lqVideoPath:", lqVideoPath);
				loadNewHDVideo(lqVideoPath, hdVideoPath, previewFilePath, streamer, streamerHD);
			}
			
			// SELECT THE FIRST ITEM inside the list if there is content list
			else
			{
				if(hasContent) mcList.clickItem(startWithItemNo - 1);
			}
		}

		public function toggleList(flag : Boolean = true) : void
		{
			mcList.visible = flag;
		}

		/*
		 * to do when an item in the list is released
		 */
		public function listItemReleaseFunction(item : Object) : void
		{
			//trace("videoHDPath:<>" + item.videoHDPath + "<>");
			// stop timer for current item
			clearTimeout(timeToStayId);
			
			// define what to do after the image is loaded and shown(work in image mode??)
			if(item.videoPath == "" && item.videoHDPath == "" && mcList.loopList) 
			{
				afterImageIsShown = function(flag : Boolean = true) : void
				{
					clearTimeout(timeToStayId);
					timeToStayId = flag ? 1 : -1;
					
					// start timer for current item if is the case
					if(flag && timeToStayId > 0) timeToStayId = setTimeout(mcList.next, item.timeToStay * 1000);
				};
			}
			else afterImageIsShown = null;
			
			
			// load the normal video and the preview
			loadNewHDVideo(item.videoPath, item.videoHDPath, item.previewPath, item.streamer, item.streamerHD);
			
			// if it has to start video hd first simulate the hd button click
//			if(mcVideoPlayer.startWithHD) releaseFunction(mcHdtoggleBt);
		}	

		
		/*----------------------------------------------
		 * 
		 * 				CONTROLLER AUTOHIDE FUNCTIONS
		 * 
		 *----------------------------------------------*/		
		
		/*
		 * enable / disable the autohide
		 */
		public function toggleAutohide(flag : Boolean = true) : void
		{
			if(flag)
			{
				if(controllerAutohideTime > 0)
				{ 
					showController();
					stage.addEventListener(MouseEvent.MOUSE_MOVE, componentMoveOverHandler);
					stage.addEventListener(Event.MOUSE_LEAVE, stageOutHandler);
				}
			}
			else
			{
				stage.removeEventListener(MouseEvent.MOUSE_MOVE, componentMoveOverHandler);
				stage.removeEventListener(Event.MOUSE_LEAVE, stageOutHandler);
            	
				clearTimeout(autoHideTimeOutId);						
			}
		}

		/**
		 * changes the state of the stage : fullscreen / normal
		 */
		private function componentMoveOverHandler(event : Event) : void 
		{
			// show the controller
			if(mcBack.hitTestPoint(stage.mouseX, stage.mouseY)) showController();
			// hide the controller
			else hideCPAfterTime();
		}

		/*
		 * changes the state of the stage : fullscreen / normal
		 */
		private function stageOutHandler(event : Event) : void 
		{
			// hide the controller
			hideCPAfterTime();
		}

		/**
		 * shows / hides the controller depending on the flag
		 */
		public function showController(flag : Boolean = true) : void
		{
			if(controllerAutohideTime > 0)
			{
				// call the external listener
				if(hideListener != null) 
				{
					if(hideListener.length) hideListener(flag, controllerShowTime);
					else hideListener();
				}			
				
				// change alpha for the controller and HD button
				Tweener.addTween(mcController, {alpha : (flag == true ? 1 : 0), time: controllerShowTime});
				if(hasContent) Tweener.addTween(mcList, {alpha : (flag == true ? 1 : 0), time: controllerShowTime});
				
	
				// after a time hide it again if is the case to show
				if(flag)
				{
					clearTimeout(autoHideTimeOutId);
					autoHideTimeOutId = setTimeout(hideCPAfterTime, controllerAutohideTime * 1000);
				}		
			}	
		}

		/*
		 * hides the controller after the time has pased if is the case
		 */
		public function hideCPAfterTime() : void
		{
			if(!mcController.hitTestPoint(stage.mouseX, stage.mouseY, true) && isPlaying) showController(false);
		}	

		
		
		/*----------------------------------------------
		 * 
		 * 				RESIZE FUNCTIONS
		 * 
		 *----------------------------------------------*/
		/*
		 * changes the state of the stage : fullscreen / normal
		 */
		public function fullScreenHandler(event : Event = null) : void 
		{
			var tempWidth : Number;
			var tempHeight : Number;
			// make invisible the others and reposition the player
			if(stage.displayState == StageDisplayState.FULL_SCREEN)
			{
				// change the status
				this.isFullScreen = true;
								
				// reposition recursively to (0, 0) coods
				fullPlayerXPos = this.x;
				fullPlayerYPos = this.y;
				fullPosition(this);
				this.x = fullPlayerXPos;
				this.y = fullPlayerYPos;
				
				// resize dimensions
				tempWidth = stage.stageWidth;
				tempHeight = ((controllerUnder) ? stage.stageHeight - controllerHeight : stage.stageHeight);
			}
			// NORMAL STATE 
			else 
			{
				// reposition the player to the initial coords
				this.x = initPlayerXPos; 
				this.y = initPlayerYPos;
				
				// resize dimensions
				tempWidth = componentWidth;
				tempHeight = componentHeight;
				
				// change the status
				this.isFullScreen = false;
				
				// show the controller
				if(controllerAutohideTime > 0) showController(true); // trace("show controller");
			}
			
			// make the resize depending on the new dimensions
			resize(tempWidth, tempHeight);
			
			// restart adjusting the progress bar
			//mcVideo.setProgress();
			
			// change the fullbt
			mcController.changeFullBt();
		}

		/*
		 * calculates recursively the position of the player in full state
		 */
		private function fullPosition(mc : Object) : void
		{
			if(mc.parent != null)
			{
				fullPlayerXPos -= mc.x;
				fullPlayerYPos -= mc.y;
				fullPosition(mc.parent);
			}
		}

		/*
		 * resizes everything when changing to full / normal screen
		 */
		public function resize(pWidth : Number = NaN, pHeight : Number = NaN) : void
		{
			// resize Background
			mcBack.width = pWidth;
			mcBack.height = pHeight;
			
			// loader
			mcLoader.centerOnClip(mcBack);
			mcLoader.x = Math.round((pWidth - mcLoader.width) / 2); //trace(mcLoader.x);
			mcLoader.y = Math.round((pHeight - mcLoader.height) / 2); //trace(mcLoader.y);
			
			// video button
			mcVideoButton.x = mcVideoButton.y = 0;
			mcVideoButton.width = mcBack.width; 
			mcVideoButton.height = mcBack.height; //trace("Video button height: " + mcVideoButton.height);
			
			// reposition Center button
			mcCenterPlayBt.centerOnClip(mcBack);
			
			// resize imageHolder
			if(previewLoaded) mcImageHolder.fitToRect(pWidth, pHeight);
			
			// resize videoHolder
			mcVideo.x = mcVideo.y = 0;
			mcVideo.resize(pWidth, pHeight - (controllerUnder ? controllerHeight : 0));
	    	
			// resize & reposition Controller
			mcController.x = controllerMargin;
			mcController.y = pHeight - (!controllerUnder ? controllerHeight : 0) - controllerMargin;
			mcController.resize(pWidth - 2 * controllerMargin, controllerHeight);	    	
	    	
			// PLAYLIST
//			mcList.resize(mcBack.width);
			mcList.x = Math.round((mcBack.width - mcList.mcBack.width) / 2);
			mcList.y = mcController.y - mcList.mcBack.height - controlsDistanceBetween;
			
			// reposition the logo
			if(mcLogo.loaded) repositionLogo();
			
			// resize share window
			if(mcController.mcEmbedBt.hidden) mcEmbed.y = mcBack.y + mcBack.height;
			mcEmbed.resize(pWidth, pHeight);
			
			// message
			resizeErrorMessage(pWidth, pHeight);
			
			// resize bars
			setProgress();			
			setDownload();
		}

		/*
		 * resize the error message
		 */
		public function resizeErrorMessage(pWidth : Number, pHeight : Number) : void
		{
			// background
			mcMessage.mcBack.width = pWidth;
			mcMessage.mcBack.height = pHeight;
			
			// sign
			mcMessage.mcSign.x = mcMessage.mcBack.x + Math.round((mcMessage.mcBack.width - mcMessage.mcSign.width) / 2);			
			mcMessage.mcSign.y = mcMessage.mcBack.y + Math.round((mcMessage.mcBack.height - mcMessage.mcSign.height) / 2);
			
			// text
			mcMessage.mcText.x = mcMessage.mcBack.x + Math.round((mcMessage.mcBack.width - mcMessage.mcText.width) / 2);			
			mcMessage.mcText.y = mcMessage.mcSign.y + mcMessage.mcSign.height + 10; //mcMessage.mcBack.y + Math.round((mcMessage.mcBack.height - mcMessage.mcText.height) / 2);				
		}

		/*
		 * repositions the logo on the new dimensions
		 */
		public function repositionLogo() : void
		{
			var pWidth : Number = mcBack.width;
			var pHeight : Number = mcBack.height;
			
			// x position
			if(logoPosition == "TL" || logoPosition == "BL") mcLogo.x = logoMargin;
			else mcLogo.x = pWidth - mcLogo.width - logoMargin;
			// y position
			if(logoPosition == "BL" || logoPosition == "BR") mcLogo.y = pHeight - mcLogo.height - controllerHeight - logoMargin;
			else mcLogo.y = logoMargin;
		}		



		/*----------------------------------------------
		 * 
		 * LOADS a new video inside the player(could be called from a list item)
		 * 
		 *----------------------------------------------*/
		/*
		 * loads a video with the hd version too inside the player
		 */
		public function loadNewHDVideo(pVideoPath : String = "", pVideoHDPath : String = "", 
									   pPicturePath : String = "", 
									   pStreamer : String = null, pStreamerHD : String = null,
									   pDuration : String = "", 
									   pStartWithHD : Boolean = false) : void
		{
			// STOP EVERYTHING
			stopAction(true);
			
			// set the hd path
			hdVideoPath = pVideoHDPath;
			streamerHD = pStreamerHD;
	    	
			// set the lq path and streamer
			lqVideoPath = pVideoPath;
			streamerLQ = pStreamer;
	    	
			// set the preview path
			previewFilePath = pPicturePath;
			
			// parse duration
			if(Prototypes.readStringVar(pDuration) == "") duration = NaN;
			else duration = Prototypes.parseTimeToSeconds(pDuration);
//			trace("duration:", duration, pDuration);
			
			// activate or not the hd button
			mcController.disableHD(hdVideoPath == "");
			
			// test if is an audio
			startWithHD = pStartWithHD;
			if(lqVideoPath != null && lqVideoPath.indexOf(".mp3") >= 0) startWithHD = false;
	    	
	    	// start the video
			if(startWithHD && hdVideoPath != "") loadHQVideo();
	    	else loadLQVideo();
		}	
		
		/*
		 * load the low quality video inside the player
		 */
		public function loadLQVideo() : void
		{
			// change hd button
			startWithHD = false;
			mcController.writeHdBtText(hdToggleSDText);
			
			// load the video
			loadNewVideo(lqVideoPath, previewFilePath, streamerLQ);
		}

		/*
		 * load the high quality video inside the player
		 */
		public function loadHQVideo() : void
		{
			// change hd button
			startWithHD = true;
			mcController.writeHdBtText(hdToggleHDText);
			
			// load the video
			loadNewVideo(hdVideoPath, previewFilePath, streamerHD);
		}			
		
		/*
		 * LOAD new video inside the player
		 */
		public function loadNewVideo(pVideoPath : String = "", pPicturePath : String = "", pStreamer : String = null) : void
		{
			// stop and clear current VIDEO / AUDIO
			mcVideo.clearVideo();
			soundManager.stopAudio();
			setTotalTime(0);
			toggleListeners(false);
			
			// hide and clear current PREVIEW IMAGE
			if(mcImageHolder.stopLoading) mcImageHolder.stopLoading();
			mcImageHolder.showObject(false);
			
			// hide the ERROR MESSAGE if it was before
			hideErrorMessage();
			
			// show loader
//			mcLoader.visible = true;

			// (de)activate CONTROLS (if is a video or just an image) untill the video can start
			mcController.activateControls(false);
			mcController.initBars();
			mcCenterPlayBt.visible = false;
			
			
			// read PARAMS
			videoFilePath = Prototypes.readStringVar(pVideoPath);
			streamer = Prototypes.readStringVar(pStreamer, null);
			autoPlay = (firstPlay ? autoStartPlayer : autoStart);
//			trace("autoPlay:", autoPlay, autoStartPlayer, autoStart);		
			
			
			// LOAD THE PREVIEW if not youtube
			if(videoFilePath.indexOf("youtube.com") < 0) loadPreviewImage(pPicturePath);
			else 
			{
//				loadPreviewImage("http://img.youtube.com/vi/7Nr6E3sPOdE/3.jpg");
//				mcVideo.visible = true;			
			}
			
			// change first load status
			if(firstPlay) firstPlay = false;			
			
			// LOAD THE MEDIA
			if(videoFilePath) 
			{
				toggleListeners();
				
				// load the new audio
				if(isPlayingAudio) soundManager.loadNewSound(videoFilePath, autoPlay);
				// load the new video
				else mcVideo.loadNewVideo(videoFilePath, streamer, autoPlay);
			}
		}
		
		/*
		 * TO DO when media has been loaded and can start
		 */		
		public function mediaLoadedListener(e : Event) : void
		{
//			trace("VIDEO LOADED! Can start", e.currentTarget);
			// hide loader
			mcLoader.visible = false;
			
			// show center button
			if(!isPlaying && !disableCenterPlayBt) mcCenterPlayBt.visible = true;			
			
			// enable controls
//			mcController.activateControls();
			
			// START THE NEW VIDEO ONLY IF IS TO AUTOSTART
			if(autoPlay && !isPlaying) 
			{
//				trace("START");
				playAction();
			}
			else
			{
//				trace("DO NOT START");
				mcController.mcPlayBt.mouseEnabled = true;
				mcController.mcPlayBt.alpha = 1;
			}
		}
		
		/*
		 * TO DO when media has been the buffer full and can start progress monitor
		 */	
		public function mediaBufferFullListener(e : Event) : void
		{
//			trace("start monitor");

			if(isPlaying)
			{
				// enable controls
				mcController.activateControls();
	
				// start showing the playing progress on the controller
				toggleProgress();
			}
		}
		
		/*
		 * TO DO when media has started playing
		 */
		public function mediaPlayingStartListener(e : Event = null) : void
		{
    		// dispatch event
    		dispatchEvent(new Event("MediaPlayingStart"));				
		}
		
		/*
		 * TO DO when media has started playing for the first time
		 */
		public function mediaPlayingPauseListener(e : Event = null) : void
		{
    		// dispatch event
    		dispatchEvent(new Event("MediaPlayingPause"));				
		}	
		
		/*
		 * TO DO when media has ended playing
		 */
		public function mediaPlayingEndListener(e : Event = null) : void
		{
//			trace("MEDIA ENDED IN PLAYER", getPlayedTime, getTotalTime, getTotalTime - 0.05);
			if(!mcController.tracking && getPlayedTime >= getTotalTime - 0.05)
	    	{
	    		if(hasContent)
	    		{
	    			if(mcList.loopList) mcList.next();
	    			else mcController.releaseFunction(mcController.mcStopBt);
	    		}
	    		else 
	    		{
//	    			mcController.releaseFunction(mcController.mcStopBt);
	    		}
	    		
	    		// dispatch event
	    		dispatchEvent(new Event("MediaPlayingEnd"));	    		
	    	}			
		}		
		
		/*
		 * TO DO when media has stopped playing
		 */
		public function mediaPlayingStopListener(e : Event = null) : void	
		{
    		// dispatch event
    		dispatchEvent(new Event("MediaPlayingStop"));						
		}

		/**
		 * loads the preview image
		 * @param pPicturePath
		 */
		public function loadPreviewImage(pPicturePath : String) : void
		{
			// retain parameter
			previewFilePath = pPicturePath;
			
			// reposition
			mcImageHolder.x = mcImageHolder.y = 0;
			
			// hide image
			mcImageHolder.showObject(false, 0);
			
			// show the loader & hide CENTER PLAY BUTTON if the player should not autostart the video
			var tempAutoStart : Boolean = (firstPlay ? autoStartPlayer : autoStart);
			if(!tempAutoStart) 
			{
//				mcLoader.visible = true;
				mcCenterPlayBt.visible = false;
			}
			
			// load and show the new image
			previewLoaded = false;
			mcImageHolder.onIOErrorFunction = previewLoadedFunction;
			mcImageHolder.loadContent(previewFilePath, previewLoadedFunction, null, false); 
		}

		/*
		 * to do after the preview image is loaded
		 */
		private function previewLoadedFunction(currentHolder : MovieClip = null) : void
		{
			// change the status
			previewLoaded = true;
		
			// hide the loader
			mcLoader.visible = false;				
			
			// resize and reposition the holder to the video dimensions
			mcImageHolder.fitToRect(mcBack.width, mcBack.height); 
			
			// start showing the preview image if the player is stopped or is playing an audio
			if(!isPlaying || isPlayingAudio) 
			{
//				trace("PICTURE LOADED:", currentHolder.name);
				mcImageHolder.alpha = 0;
				mcImageHolder.showObject(true, 0.5, function() : void 
				{
					picsMotionFinished(mcImageHolder);
				});
			} 
		}

		/*
		 * to do after the preview is shown(fade in)
		 */
		public function picsMotionFinished(mc : MovieClip) : void
		{
			// show the center play button
			var tempAutoStart : Boolean = (firstPlay ? autoStartPlayer : autoStart);
			if(!tempAutoStart && !isPlaying)
			{ 
//				trace("arat center play", mcImageHolder.alpha, mcImageHolder.visible);
				if(showPlayFlag && !disableCenterPlayBt) mcCenterPlayBt.visible = true;
			}
			
			// call the after shown function
			if(afterImageIsShown != null) afterImageIsShown();
		}
		
		/*
		 * show the preview instantly
		 */
		public function showFastPreview(flag : Boolean) : void
		{
			var goToAlpha : Number = ((flag == true) ? 1 : 0);
			Tweener.removeTweens(mcImageHolder);
			mcImageHolder.alpha = goToAlpha;
		}		
		
		
		/*
		 * to do after the logo is loaded
		 */
		private function logoLoaded(currentHolder : MovieClip) : void
		{
			currentHolder = mcLogo;
			
			// change the status
			currentHolder.loaded = true;						

			// reposition the logo
			repositionLogo();
			currentHolder.showObject();
		}	
		
		/*
		 * show error window
		 * flag = true - > the path is not found
		 * flag = false - > the connection failed
		 */
		public function showErrorMessage(errorFlag : Boolean = true, flag : Boolean = true) : void
		{
			// show
			if(flag)
			{
				// write the text
				var message : String = "Connection failed. Please refresh the page.  ";
				if(errorFlag) message = "Stream not found" + ((videoFilePath && videoFilePath != "") ? (": " + videoFilePath) : ("."));
				mcMessage.mcText.txt.text = message;
				
				// resize the message box
				resizeErrorMessage(mcMessage.mcBack.width, mcMessage.mcBack.height);
				
				// press the stop button
				mcController.releaseFunction(mcController.mcStopBt);
			}			
			
			// show / hide the message window
			mcMessage.visible = flag;
		}
		public function hideErrorMessage() : void
		{
			mcMessage.visible = false;
		}				


		
		/********************************************************
		 * 
		 * 					PLAY FUNCTIONS
		 * 
		 ********************************************************/		
		
		/**
		 * START PLAYING
		 */
		public function playAction() : void
		{
			mcController.releaseFunction(mcController.mcPlayBt); // this will call the function below
		}
		
		public function playVideo() : void
		{
			// start showing the playing progress on the controller
//			toggleProgress();	
			
			// start showing the download progress on the controller
			toggleDownload();				
			
			// start playing
			// if is a regular mp3(not streamed via RTMP)
//			trace("streamer:", streamer);
			if(lqVideoPath != null && lqVideoPath.indexOf(".mp3") >= 0 && streamer == null) soundManager.playAudio();
			else mcVideo.playVideo();
		}	
		
		
		/**
		 * PAUSE PLAYING
		 */		
		public function pausePlaying() : void
		{
			mcController.releaseFunction(mcController.mcPauseBt); // this will call the function below
		}
		public function pauseVideo() : void
		{
			// pause progress
			toggleProgress(false);
			
			// pause media
			if(lqVideoPath.indexOf(".mp3") >= 0) soundManager.pauseAudio();
			else mcVideo.pauseVideo();
		}				

		/**
		 * STOP PLAYING
		 * stops the video and changes the controls
		 */
		public function stopAction(clearFlag : Boolean = false) : void
		{
			mcController.releaseFunction(mcController.mcStopBt);
			
			// stop progress
			toggleProgress(false);			
			
			// stop download
			toggleDownload(false);
			
			if(clearFlag)
			{
				videoFilePath = "";
				streamer = null;
				previewFilePath = "";
			}		
		}
		
		/*
		 * stops just the video
		 */
		public function stopVideo() : void
		{
			// stop media
			if(isPlayingAudio) soundManager.stopAudio();
			else mcVideo.stopVideo();
		}		
		
		/**
		 * SEEK PLAYING
		 */
		public function seekVideoToProcent(percent : Number) : void
		{
			if(isPlayingAudio) 
			{
//				trace("seek to:", percent);
				soundManager.seekAudioToProcent(percent);
			}
			else 
			{
//				mcVideo.seekVideoToProcent(percent);
				mcVideo.seekVideoToTime(percent * getTotalTime);
			}
		}		


		/*
		 * PROGRESS FUNCTIONS
		 */		
		public function toggleProgress(flag : Boolean = true) : void
		{
			clearTimeout(progressIntervalId);
			if(flag) progressIntervalId = setInterval(setProgress, 10);
		}
		
	    public function setProgress() : void 
	    {
			// set playing progress on the controller
			mcController.setProgress(getPlayedPercent);
		}
		
		
		/*
		 * DOWNLOAD FUNCTIONS
		 */
		public function toggleDownload(flag : Boolean = true) : void
		{
			clearTimeout(downloadIntervalId);
			if(flag) downloadIntervalId = setInterval(setDownload, 10);
		}
		
	    public function setDownload() : void 
	    {
			// set download progress
			mcController.setDownload(isPlayingAudio ? soundManager.getLoadedPercent : mcVideo.getLoadedPercent);
		}


		/********************************************************
		 * 
		 * 					SETTERS / GETTERS
		 * 
		 ********************************************************/
		 
		public function get isVideoMode() : Boolean
		{
			return (mode == VIDEOMODE);
		}			 
		 
		/*
		 * SET played time in timer
		 */
		public function setPlayedTime(procent : Number) : void
		{
			mcController.mcTimer.mcSign.tPassedTime.htmlText = Prototypes.videoTimeConvert(procent * getTotalTime);
			mcController.mcTimer.mcSign.tPassedTime.setSize(timerTextSize);
		}
		
		/*
		 * SET total time in timer
		 */		
		public function setTotalTime(pTime : Number = NaN) : void
		{
			mcController.mcTimer.mcSign.tTotalTime.htmlText = Prototypes.videoTimeConvert(!isNaN(pTime) ? pTime : getTotalTime);
			mcController.mcTimer.mcSign.tTotalTime.setSize(timerTextSize);			
		}
		
		/*
		 * GET total time based on the type of media played
		 */
		public function get getTotalTime() : Number
		{
			return (!isNaN(duration) ? duration : (isPlayingAudio ? soundManager.getTotalTime : mcVideo.getTotalTime));	
		}		


		/*
		 * Get the played time
		 */
		public function get getPlayedTime() : Number
		{
			return (isPlayingAudio ? soundManager.getPlayedTime : mcVideo.getPlayedTime);
		}		
		
		public function get getPlayedPercent() : Number
		{
//			trace(getPlayedTime, getTotalTime);
			return (getTotalTime > 0 ? (getPlayedTime / getTotalTime) : 0);
		}	



		/*
		 * GET volume from controller
		 */		
		public function get getVolume() : Number
		{
			return mcController.getVolume;
		}		

		/*
		 * SET volume with fade
		 */		
		public function setVolume(value : Number) : void
		{
//			trace("volume:", value);
			setVolumeInstant(value, fadeSoundTime);
		}
		// set volume instantly
		public function setVolumeInstant(value : Number, fadeTime : Number = 0) : void
		{
//			if(isPlayingAudio) 
			soundManager.setVolumeInstant(value, fadeTime);
//			else 
			mcVideo.setVolumeInstant(value, fadeTime);		
		}		
				

		/*
		 * GET the playing status
		 */
		public function get isPlaying() : Boolean
		{
			return (isPlayingAudio ? soundManager.isPlaying : mcVideo.isPlaying);
		}
		
		/*
		 * GET what type of media is playing status
		 */		
		public function get isPlayingAudio() : Boolean
		{
			return (videoFilePath.indexOf(".mp3") >= 0 && streamer == null);
		}		
		



		/********************************************************
		 * 
		 * 					EVENT HANDLERS
		 * 
		 ********************************************************/
		 
		/**
		 * to do on PRESS a button
		 */
		private function pressHandler(event : MouseEvent) : void
		{
			// set pressed this item
			this.currPressedItem = MovieClip(event.currentTarget);
		}

		/*
		 * to do on RELEASE a button 
		 */
		private function releaseHandler(event : MouseEvent) : void
		{
			var mc : MovieClip = MovieClip(event.currentTarget);
			
			// test if the event came from the same pressed item
			if(mc == this.currPressedItem)
			{
				if(mc == mcLogo) URLNavigator.ChangePage(logoURL, "_blank");
			}
		}

		
		
		
		
		/********************************************************
		 * 
		 * 					AUXILIAR FUNCTIONS
		 * 
		 ********************************************************/
		public function get hasContent() : Boolean
		{	
			return (contentXmlObject != null && contentXmlObject.children().length() > 0);	 
		}
		/**
		 * hides some controls when the player is used as image viewer
		 */
		public function hideControls(flag : Boolean) : void
		{
			mcController.mcPlayBt.visible = !flag;
			mcController.mcPauseBt.visible = !flag;
			mcController.mcStopBt.visible = !flag;
			mcController.mcPlayBar.visible = !flag;
			mcController.mcTimer.visible = !flag;
			mcController.mcVolumeCt.visible = !flag;
			showPlayFlag = !flag;
		}

		/*
		 * repositions and show the loader before loading the xml files
		 */
		public function reposLoader() : void
		{
			mcLoader.centerOn(!isNaN(componentWidth) ? componentWidth : stage.stageWidth, !isNaN(componentHeight) ? componentHeight : stage.stageHeight);
//			mcLoader.visible = true;
		}		

		/**
		 * show / hide the elements inside the component 
		 */
		public function show(flag : Boolean = true, loaderFlag : Boolean = false) : void
		{
			mcLoader.visible = (loaderFlag ? flag : !flag);
			//
			mcController.visible = flag;
			//
			mcVideoButton.visible = flag;
			mcVideo.visible = flag;
			//
			mcBack.visible = flag;			
		}

		/*
		 * downloads the current shown media in a pop-up
		 */
		public function downloadMedia() : void 
		{
			var windowTarget : String = "_self"; //"_blank";//
			var windowUrl : String = downloadScript + "?f=" + (videoFilePath != "" ? videoFilePath : previewFilePath);
			URLNavigator.ChangePage(windowUrl, windowTarget);
		}

		
	} // FROM CLASS
} // FROM PACKAGE