﻿/**
 * Copyright (c) 2009 Vuzum LLC <http://www.vuzum.com/>
 */

package com.vuzum.mediaplayer
{
	import flash.events.EventDispatcher;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.events.StatusEvent;
	import flash.media.ID3Info;
	import flash.media.Sound;
	import flash.media.SoundChannel;
	import flash.media.SoundLoaderContext;
	import flash.media.SoundTransform;
	import flash.net.URLRequest;
	import flash.utils.*;

	// OTHER IMPORTS
	import caurina.transitions.*;

	// VUZUM IMPORTS 
	import com.vuzum.utils.*;

	/*
	 * CLASS
	 */
	public class SoundManager extends EventDispatcher
	{
		// INTERNAL SOUND OBJECTS
		private var sound : Sound;
		private var sc : SoundChannel;
		private var st : SoundTransform;
		private var songInfo : ID3Info;
		
		
		// SETTINGS VARS
		// song info
		private var currSoundPath : String;
		private var currentSongName : String = "";		
		// playing vars
		private var pos : Number = 0;
		public var isPlaying : Boolean = false;
		private var soundHasBeenSet : Boolean = false;
		//
		private var autoStart : Boolean = false;
		private var repeat : Boolean = false;
		private var bufferTime : Number = 0;
		// volume vars
		public var fadeSoundTime : Number = 0;
		public var currentVolume : Number = 0;
		

		// rewind / forward time
		private var rewindTime : Number = 8;//seconds
		private var forwardTime : Number = 30;
		
		// time strings
		private var totalTime : String;
		private var passedTime : String;
		
		
		
		// AUX VARS
		// from this time to this time the progress actualizes
		private var checkEndIntervalId : uint;		
		private var bufferCheckID : uint;
			
		
		
		/**
		 * CONSTRUCTOR
		 */
		public function SoundManager() 
		{
			// Prototypes
			new Prototypes();
			
			// init sound Objects
			st = new SoundTransform();
			sc = new SoundChannel();
		}
		
		/*******************************************
		 * 
		 * 				PLAY FUNCTIONS
		 * 			
		 *******************************************/

		/*
		 * loads a new sound in the player and starts it if is autoplay
		 */
		public function loadNewSound(soundPath : String, pAutostart : Boolean = false, pRepeat : Boolean = false) : void
		{
			// stop everyting
			stopAudio();
			
			// read params
			currSoundPath = soundPath; //trace("currPath:", currPath);
			autoStart =	pAutostart;		
			repeat = pRepeat;
			
			// reset vars
			setIsSoundSet = false;
			
			// load the new param sound
			if(currSoundPath != "")
			{
				// PREPARE THE SOUND
				sound = new Sound();
				sound.addEventListener(Event.ID3, loadSongMetadata);
				sound.addEventListener(flash.events.StatusEvent.STATUS, function() : void
				{
//					trace("the song started");
				});			
				sound.addEventListener(IOErrorEvent.IO_ERROR, function(e : Event) : void
				{
					//FIXME: Error loading song
//					trace("The song could not start!");
				});
				
				// tell the player the audio can start
				dispatchEvent(new Event("MediaLoaded", true, false));				
			}
			else
			{
				//FIXME: Error Empty audio path!
			}
		}
		
		/**
	     * PLAY for the first time or RESUME current audio
	     */
	    public function playAudio() : void 
	    {
	    	var req : URLRequest = new URLRequest(currSoundPath);
			var context : SoundLoaderContext = new SoundLoaderContext(this.getBufferTime * 1000, true);
			
			// play for the first time
			if(!isSoundSet)
			{
//				trace("START", currSoundPath);
				// change the status
				setIsSoundSet = true;
				
		    	// change the play status
				isPlaying = true;				
				
				// load sound
				sound.load(req, context);
			
				// start playing new song
				sc = new SoundChannel();
				playAtPos();
			}
			// resume
			else 
			{
//				trace("RESUME", currSoundPath);
				resumeAudio();
			}
			
			// tell the player that the sound started
			dispatchEvent(new Event("MediaPlayingStart"));			
		}
		
		private function playAtPos(ppos : Number = 0) : void
		{
			// stop current playing
			sc.stop();
			
			// retain current position
			pos = ppos;
			
			// start palying
			sc = sound.play(pos);
			sc.addEventListener(Event.SOUND_COMPLETE, soundComplete);
		
			// set current volume
			setVolumeInstant(currentVolume);
			
	    	// start checking the buffer status
	    	toggleCheckBuffer();						
		}		

		/*
		 * checks buffering to start showing the progress status after the buffer is full
		 */
		private function toggleCheckBuffer(flag : Boolean = true) : void
		{
	    	clearInterval(bufferCheckID);
			if(flag) bufferCheckID = setInterval(checkBuffering, 10);			
		}
		private function checkBuffering() : void
		{
//			trace("isBuffering:", sound.isBuffering);
			if(!sound.isBuffering)
			{
				toggleCheckBuffer(false);
				
	//			dispatchEvent(new Event("MediaLoadingStarted", true, false));
	
				// tell the player that the video is loaded and can start progres monitor
				dispatchEvent(new Event("MediaBufferFull"));
		    	
	            // start check end interval
	            toggleCheckEnd();
			}
		}
		
		/**
	     * RESUME current audio
	     */
	    public function resumeAudio() : void 
	    {
			// change the action	
			isPlaying = true;
			
			// resume playing
			playAtPos(pos);
		}
		
		/**
	     * Pauses the Audio
	     */
	    public function pauseAudio() : void 
	    {
			// stop check end interval
//			toggleCheckEnd(false);	    	
	    	
			// stop playing audio
			if(sc)
			{
				pos = sc.position;
				sc.stop();
			}
			
			// change the status
			isPlaying = false;
			
			// tell the player that the sound paused
			dispatchEvent(new Event("MediaPlayingPause"));						
		}
		
		/**
	     * Stops the audio
	     */
	    public function stopAudio() : void 
	    {
			// stop check end interval
            toggleCheckEnd(false);
            
			// stop audio
			if(sc) sc.stop();
			
			// move to begin
			pos = 0;

	    	// change the status
			isPlaying = false;
			
			// arunca event ca s-a stopat sunetul
			dispatchEvent(new Event("MediaPlayingStop"));
		}
		
		// SEEK TO PERCENT
		public function seekAudioToProcent(percent : Number) : void
		{
			// retain position
			pos = percent * sound.length;
			
			// resume playing
			if(isPlaying) playAtPos(pos);
		}		
		
		// REWIND
		public function rewind() : void
		{
			// PAUSE AUDIO
			sc.stop();			
			
			// CALCULATE THE NEW POSITION
			pos -= this.getRewindTime * 1000;
			pos = (pos < 0 ? 0 : pos);
			
			// resume playing
			if(isPlaying) playAtPos(pos);
		}
		// FORWARD
		public function forward() : void
		{
			// PAUSE AUDIO
			sc.stop();
			
			// CALCULATE THE NEW POSITION
			pos += this.getForwardTime * 1000;
			
			// UPDATE the TIMER
			pos = (pos > sound.length ? sound.length : pos);
									
			// resume playing
			if(isPlaying) playAtPos(pos);
		}
		
		
		/*
		 * loads the mp3 information in the textfield
		 */
		private function loadSongMetadata(e : Event) : void
		{
			// READ THE INFO object
			songInfo = ID3Info(e.target.id3); 
			
			// WRITE THE INFO in the textfield
         	this.setCurrentSongName = "";
			if(songInfo.artist == null || songInfo.artist == "")
				 this.setCurrentSongName = "Unknown Artist";
			 else 
				 this.setCurrentSongName = songInfo.artist;
			//
			if (songInfo.songName == null || songInfo.songName == "" )
				 this.setCurrentSongName = this.getCurrentSongName + " - Unknown";
			 else 
				 this.setCurrentSongName = this.getCurrentSongName + " - " + songInfo.songName;				 
		}		
		
		
        // start check end interval
        public function toggleCheckEnd(flag : Boolean = true) : void
        {
            clearInterval(checkEndIntervalId);
            if(flag) checkEndIntervalId = setInterval(checkEnd, 250);
        }		
		public function checkEnd() : void
		{
			if((sound.bytesLoaded > 0 && sound.bytesTotal > 0 && sound.bytesLoaded == sound.bytesTotal) && (sc.position >= sound.length))
			{
				// stop audio or play next song depending on loop variable
				soundComplete();			
			}			
		}
		/*
		 * sound completed listerner
		 */
		private function soundComplete(e : Event = null) : void 
		{
//			trace("audio ended");
			dispatchEvent(new Event("MediaPlayingEnd"));
		}

		
		
		//_____________SETTERS / GETTERS_____________
		/**
		 * Set the total time string
		 */
		public function set setTotalTimeString(totalTimeString : String) : void
		{
			totalTime = totalTimeString;
		}
		
		/**
		 * Set the total time string
		 */
		public function set setPassedTimeString(passedTimeString : String) : void
		{
			passedTime = passedTimeString;
		}
		
		/**
		 * Get the percentage loaded for the sound
		 */
		public function get getPercentLoaded() : Number
		{
			return (sound.bytesTotal > 0 ? (sound.bytesLoaded / sound.bytesTotal) : 0);
		}		
		
		
		/**
	     * Sets the audio's volume
	     * @param pVolume the new volume's value
	     */
	    public function set setVolume(pVolume : Number) : void
	    {
	    	 setVolumeInstant(pVolume, fadeSoundTime);
	    }
	    public function setVolumeInstant(pVolume : Number, fadeTime : Number = 0) : void 
	    {
			Tweener.addTween(st, { volume:pVolume, time: fadeTime, transition : "easeOutQuad", onUpdate : function() : void 
			{
				currentVolume = st.volume;
				
				if(sc != null) sc.soundTransform = st;
				else
				{
					//FIXME: ERROR Sound Channel is null
//					trace("Sound Channel is null!");
				}
			}});
	    }
	    
		/**
	     * Get current audio's volume
	     */
	    public function get getVolume() : Number 
	    {
	        return currentVolume;
		}	    
	    
	    public function set setFadeTime(value : Number) : void
	    {
			fadeSoundTime = value;
		}
	    
	    
	    /**
	     * Set the buffer time
	     */
	    public function set setBufferTime(pbufferTime : Number) : void 
	    {			
			bufferTime = pbufferTime;
		}
		
		/**
	     * Get current buffer time
	     */
	    public function get getBufferTime() : Number 
	    {
	        return bufferTime;
		}		
		
		
		
	    /**
	     * Set the buffer time
	     */
	    public function set setRewindTime(pRewindTime : Number) : void 
	    {			
			rewindTime = pRewindTime;
		}
		
	    /**
	     * Set the buffer time
	     */
	    public function set setForwardTime(pForwardTime : Number) : void 
	    {			
			forwardTime = pForwardTime;
		}
		
		/**
		 * Set the percentage loaded for the sound
		 */
		private function set setCurrentSongName(pCurrentSongName : String):void
		{
			currentSongName = pCurrentSongName;
		}
		
		private function set setIsSoundSet(val: Boolean) : void
		{
			soundHasBeenSet = val;
		}
		
		//_____________GETTERS_____________
		/**
		 * Get the percentage loaded for the sound
		 */
		public function get getPlayedPercent() : Number
		{
//			trace(sc.position, sound.length);
			var percent : Number = (sc.position / sound.length);
			if(isNaN(percent)) percent = 0;
			
			return percent;
		}		
		
		/**
		 * Get the played time in seconds
		 */
		public function get getPlayedTime() : Number
		{
			return (sc.position / 1000);
		}

		/**
		 * Get the played time in the format MM:SS
		 */
		public function get getPlayedTimeString():String
		{
			return passedTime;
		}		

		/**
		 * Get total time
		 */		
		public function get getTotalTime() : Number
		{
			return (sound ? sound.length / 1000 : 0);
		}		
		
		/**
		 * Get the total time in the format MM:SS
		 */
		public function get getTotalTimeString():String
		{
			return totalTime;
		}
		
		
		/**
		 * Get the percentage loaded for the sound
		 */
		public function get getLoadedPercent() : Number
		{
			return (sound && sound.bytesTotal > 0 ? (sound.bytesLoaded / sound.bytesTotal) : 0 );
		}		
		
		
		/**
	     * Set the buffer time
	     */
	    private function get getRewindTime() : Number 
	    {			
			return rewindTime;
		}
		
	    /**
	     * Set the buffer time
	     */
	    private function get getForwardTime() : Number 
	    {			
			return forwardTime;
		}
		
		/**
		 * Get song name
		 */
		public function get getCurrentSongName():String
		{
			return currentSongName;
		}		
		
		/*
		 * get to know if the sound is prepaired
		 */
		public function get isSoundSet() : Boolean
		{
			return soundHasBeenSet;
		}
		
	} // FROM CLASS
}// FROM PACKAGE