﻿/**
 * Copyright (c) 2011 Vuzum LLC <http://www.vuzum.com/>
 */

package com.vuzum.mediaplayer
{
	import flash.utils.*;	
	import flash.media.SoundTransform;	
	import flash.media.SoundChannel;	
	import flash.media.Sound;	
	import flash.events.MouseEvent;	
	import flash.events.Event;	
	import flash.display.MovieClip;
	
	// OTHER'S IMPORTS
	import caurina.transitions.*;
	import caurina.transitions.properties.*;

	// MY IMPORTS
	import com.vuzum.utils.*;
	
	// CLASS
	public class VolumeControl extends MovieClip
	{
		// COMPONENTS INSIDE
		public var mcVolumeBt : MovieClip;
		//
		public var mcMask : MovieClip;
		public var mcVolumeBar : MovieClip;
		//
		public var mcBack : MovieClip;

		// COMPONENT'S VARS
		private var extSetSoundFunction : Function;
		private var verticalVolume : Boolean = false;
		private var initVolumePercent : Number = 0.5;
		
		// SETTINGS VARS
		// back settings
		private var backFillColor : Number = 20;
		private var backStrokeColor : Number = 20;
		
		// bt settings
		private var controlDimension : Number = 20;
		private var btDisable : Boolean = false;
		//
		public var btFillColor : Number;
		public var btStrokeColor : Number;
		//
		public var btSignNormalColor : Number;
		public var btSignOverColor : Number;
		private var btSignOffColor : Number;
		
		// bars settings
		private var volumeBarWidth : Number = 50;
		private var volumeBarHeight : Number = 10;
		private var levelBarColor : Number;
		private var backBarColor : Number;
		private var barsBackFillColor : Number;
		private var barsBackStrokeColor : Number;
		private var barPlusArea : Number = 3;		
		
		// scrub settings
		private var scrubWidth : Number;
		private var scrubHeight : Number;
		private var scrubColor : Number;		


		// INERNAL MEMBERS
		// handlers vars
		private var currentPressedObject : MovieClip;

		// sound objects
		public var sc : SoundChannel;
		public var sound : Sound;
		private var st : SoundTransform;
		
		// SETTINGS VARS
		private var spaceBetween : Number = 6;
		//
		private var overStrokeColor : Number;
		private var normalStrokeColor : Number;
		//
		private var overFillColor : Number;
		private var normalFillColor : Number;
		//
		private var normalAlpha : Number = 0.5;
		private var overAlpha : Number = 1;		
		//
		private var overTime : Number = 0.7;
		private var vertVolumeBarDistance : Number = 0;
		public var fadeSoundTime : Number = 1;
		
		// AUX VARS
		// tracking
		private var volTracking : Boolean = false;
		private var intervalId : uint;
		private var overVBar : Boolean = false;

		/**
		 * CONSTRUCTOR
		 */
		public function VolumeControl() 
		{
			// Prototypes			
			new Prototypes();
			
			// Caurina
			ColorShortcuts.init();
			TextShortcuts.init();
			
			// init sound Objects
			sound = new Sound();
			sc = new SoundChannel();
			st = new SoundTransform();
		}
		
		/*
		 * inits the volume controller based on an XML object
		 */
		public function initWithXMLObject(dimension : Number = NaN, pExtSetSoundFunction : Function = null, pVertical : Boolean = false, pInitVolumePercent : Number = NaN, xmlObject : * = null) : void 
		{
			// read the params
			extSetSoundFunction = pExtSetSoundFunction;
			verticalVolume = pVertical;
			initVolumePercent = pInitVolumePercent;
			controlDimension = 	dimension;		
			
			// read settings from XML
			if(xmlObject != null)
			{
				// back settings
				backFillColor = Prototypes.readNumberVar(xmlObject.backFillColor);
				backStrokeColor = Prototypes.readNumberVar(xmlObject.backStrokeColor);
				
				
				// volume bt settings
				if(isNaN(controlDimension)) controlDimension = Prototypes.readNumberVar(xmlObject.dimension);
				btDisable = Prototypes.readBooleanVar(xmlObject.button.@disable);
				if(isNaN(btFillColor)) btFillColor = Prototypes.readNumberVar(xmlObject.button.fillColor);
				if(isNaN(btStrokeColor)) btStrokeColor = Prototypes.readNumberVar(xmlObject.button.strokeColor);
				//
				if(isNaN(btSignNormalColor)) btSignNormalColor = Prototypes.readNumberVar(xmlObject.button.signNormalColor);
				if(isNaN(btSignOverColor)) btSignOverColor = Prototypes.readNumberVar(xmlObject.button.signOverColor);				
				btSignOffColor = Prototypes.readNumberVar(xmlObject.button.signOffColor);				
				
				// bar settings
				volumeBarWidth = Prototypes.readNumberVar(xmlObject.barsWidth);
				volumeBarHeight = Prototypes.readNumberVar(xmlObject.barsHeight);
				//
				levelBarColor = Prototypes.readNumberVar(xmlObject.levelBarColor);
				backBarColor = Prototypes.readNumberVar(xmlObject.backBarColor);
				//
				barsBackFillColor = Prototypes.readNumberVar(xmlObject.barsBackFillColor);
				barsBackStrokeColor = Prototypes.readNumberVar(xmlObject.barsBackStrokeColor);
				
				// scrub settings
				scrubWidth = Prototypes.readNumberVar(xmlObject.scrubWidth, mcVolumeBar.mcScrub.width);
				scrubHeight = Prototypes.readNumberVar(xmlObject.scrubHeight);				
				scrubColor = Prototypes.readNumberVar(xmlObject.scrubColor);				
			}
			
			// call main funtion
			main();
		}

		/*
		 * inits the volume controller based on some settings
		 */
		public function init(pExtSetSoundFunction : Function = null, pVertical : Boolean = false, 
							 pInitVolumePercent : Number = NaN, 
							 pControlDimension : Number = NaN, 
							 pVolumeBarWidth : Number = NaN, pVolumeBarHeight : Number = NaN,
							 pscrubWidth : Number = NaN, pscrubHeight : Number = NaN) : void 
		{
			// read the params
			extSetSoundFunction = pExtSetSoundFunction;
			verticalVolume = pVertical;
			initVolumePercent = pInitVolumePercent;
			
			// button settings
			controlDimension = pControlDimension;
			// bar settings
			volumeBarWidth = pVolumeBarWidth;
			volumeBarHeight = pVolumeBarHeight;
			// scrub settings
			scrubWidth = pscrubWidth;
			scrubHeight = pscrubHeight;
			//
//			normalAlpha = mcPlayer.controlsNormalAlpha;
//			overAlpha = mcPlayer.controlsOverAlpha;
//			overTime = mcPlayer.controlsOverTime;

			// call main funtion
			main();
		}
		
		/*
		 * Main function
		 */
		public function main() : void
		{
			// VOLUME BUTTON
			mcVolumeBt.mcOff.alpha = 0;
			//
			mcVolumeBt.ratio = mcVolumeBt.width / mcVolumeBt.height;
			mcVolumeBt.width = controlDimension;//mcVolumeBt.ratio * controlDimension;
			mcVolumeBt.height = controlDimension;
			//			
			mcVolumeBt.addEvents(this);
			mcVolumeBt.visible = !btDisable;	
			mcVolumeBt.mcOver.visible = false;
			
			// VOLUME BAR			
			// seek bar
			mcVolumeBar.mcSeekBar.mouseEnabled = mcVolumeBar.mcSeekBar.mouseChildren = false;
			mcVolumeBar.mcSeekBar.width = volumeBarWidth;
			mcVolumeBar.mcSeekBar.height = volumeBarHeight;
			// back bar
			mcVolumeBar.mcBackBar.width = mcVolumeBar.mcSeekBar.width;
			mcVolumeBar.mcBackBar.mcShape.height = mcVolumeBar.mcSeekBar.height;
			mcVolumeBar.mcBackBar.mcHit.height = mcVolumeBar.mcBackBar.mcShape.height + 2 * barPlusArea;
			mcVolumeBar.mcBackBar.mcHit.y = mcVolumeBar.mcBackBar.mcShape.height + 2 * barPlusArea;
			mcVolumeBar.mcBackBar.mcHit.y = (-1) * barPlusArea;
			mcVolumeBar.mcBackBar.mcHit.alpha = 0;
			mcVolumeBar.mcBackBar.y = mcVolumeBar.mcSeekBar.y = Math.round((controlDimension - mcVolumeBar.mcSeekBar.height) / 2);
			// back
			mcVolumeBar.mcBack.mcFill.width = 2 * mcVolumeBar.mcBackBar.x + mcVolumeBar.mcBackBar.width;
			mcVolumeBar.mcBack.mcFill.height = controlDimension;
			mcVolumeBar.mcBack.mcStroke.width = mcVolumeBar.mcBack.mcFill.width - 2 * mcVolumeBar.mcBack.mcStroke.x; //trace("mcVolumeBar.mcBack.mcStroke.width:", mcVolumeBar.mcBack.mcStroke.width)
			mcVolumeBar.mcBack.mcStroke.height = mcVolumeBar.mcBack.mcFill.height - 2 * mcVolumeBar.mcBack.mcStroke.y;
			// mask
			mcMask.width = controlDimension+1;
			mcMask.height = (verticalVolume ? mcVolumeBar.mcBack.width : controlDimension);
			mcMask.x = 0;
			mcMask.y = (verticalVolume ? (- Math.round(mcMask.height) - vertVolumeBarDistance) : 0);
			mcMask.visible = verticalVolume;		
			// scrub
			mcVolumeBar.mcScrub.width = scrubWidth; 
			mcVolumeBar.mcScrub.height = scrubHeight;
			mcVolumeBar.mcScrub.y = mcVolumeBar.mcSeekBar.y + Math.round((mcVolumeBar.mcSeekBar.height - mcVolumeBar.mcScrub.height) / 2);
			//
			mcVolumeBar.mcScrub.addEvents(this);
			mcVolumeBar.mcScrub.visible = (scrubWidth > 0);
			// add events						
			mcVolumeBar.mcBackBar.addEvents(this);			
			
			// BACK
			// decide if vertical
			if(verticalVolume)
			{
				mcVolumeBar.rotation = -90;
				mcVolumeBar.x = 0;
				mcVolumeBar.y = mcMask.y + 2 * mcMask.height;
				mcVolumeBar.mask = mcMask;
				//
				mcBack.y = mcMask.y;
				mcBack.mcFill.width = controlDimension;
				mcBack.mcFill.height = controlDimension + mcMask.height + vertVolumeBarDistance;
				
				// add events						
				mcVolumeBar.mcBack.addEvents(this);				
				mcBack.addEvents(this);
			}
			// horizontal
			else
			{
				mcVolumeBar.rotation = 0;
				mcVolumeBar.x = mcVolumeBt.width + vertVolumeBarDistance;
				mcVolumeBar.y = 0;
				mcVolumeBar.mask = null;
				//
				mcBack.y = 0;
				mcBack.mcFill.width = mcVolumeBar.x + mcVolumeBar.width;
				mcBack.mcFill.height = mcVolumeBar.height;
				mcBack.removeEvents(this);				
			}
			mcBack.mcStroke.width = mcBack.mcFill.width - 2 * mcBack.mcStroke.x;
			mcBack.mcStroke.height = mcBack.mcFill.height - 2 * mcBack.mcStroke.y;			
			
			// init settings
			volTracking = false;
			
			// volume bar level
			setVolume = initVolumePercent;  //trace("Start volume: " + mcVideoPlayer.startVolume);
			mcVolumeBt.volume = getVolume;			
			
			
			// SET COLORS
			setColors(backFillColor, backStrokeColor, 
					  btSignNormalColor, btSignOffColor,
					  barsBackFillColor, barsBackStrokeColor, levelBarColor, backBarColor,
					  scrubColor);
			
			// MAKE THE FIRST RESIZE
			//resize(mcBack.width, mcBack.height);			
		}
		
		/*
		 * sets the colors on the controller elements
		 */
		public function setColors(pBackFillColor : Number = NaN, pBackStrokeColor : Number = NaN, 
								  pSignColor : Number = NaN, pSignOffColor : Number = NaN,
								  pbarsBackFillColor : Number = NaN, pbarsBackStrokeColor : Number = NaN,
								  pLevelBarColor : Number = NaN, pBackBarColor : Number = NaN,
								  pScrubColor : Number = NaN) : void
		{
			// BACKGROUND
			if(pBackFillColor >= 0 && !verticalVolume) mcBack.mcFill.setColor(pBackFillColor);
			else mcBack.mcFill.visible = false;
			//
			if(pBackStrokeColor >= 0) mcBack.mcStroke.setColor(pBackStrokeColor); 
			else mcBack.mcStroke.visible = false;
			
			// button
			if(btFillColor >= 0) mcVolumeBt.mcBack.mcFill.setColor(btFillColor);
			else mcVolumeBt.mcBack.mcFill.alpha = 0;
			if(btStrokeColor >= 0) mcVolumeBt.mcBack.mcStroke.setColor(btStrokeColor);
			else mcVolumeBt.mcBack.mcStroke.alpha = 0;			
			//
			mcVolumeBt.mcSign.setColor(pSignColor);
			//
			mcVolumeBt.mcOff.setColor(pSignOffColor);
			
			// VOLUME BAR
			// back
			if(pbarsBackFillColor >= 0) mcVolumeBar.mcBack.mcFill.setColor(pbarsBackFillColor);
			else mcVolumeBar.mcBack.mcFill.alpha = (verticalVolume ? 1 : 0);
			if(pbarsBackStrokeColor >= 0) mcVolumeBar.mcBack.mcStroke.setColor(pbarsBackStrokeColor);
			else mcVolumeBar.mcBack.mcStroke.visible = false;
			// level & back bar
			mcVolumeBar.mcSeekBar.setColor(pLevelBarColor);
			mcVolumeBar.mcBackBar.setColor(pBackBarColor);
			// scrub
			mcVolumeBar.mcScrub.setColor(pScrubColor);			
		}

		/*
		 * RESIZES the player
		 */
		public function resize(w : Number = NaN, h : Number = NaN) : void
		{
			// resize the background
			mcBack.width = w;
			mcBack.height = h;
		}
		


		/****************************************************
		*
		*					EVENT HANDLERS
		*
		*****************************************************/
		
		/**
		 * OVER EVENT HANDLER
		 */
		public function overHandler(event : MouseEvent) : void 
		{
			var mc : MovieClip = event.currentTarget as MovieClip;
			switch(mc) 
			{
				case mcBack:
				// vertical volume bar
				case mcVolumeBar.mcBack:
				case mcVolumeBar.mcBackBar:
				case mcVolumeBar.mcScrub:
				// volume button
				case mcVolumeBt:
					// show the volume bar
					if(verticalVolume)
					{
						// change the status
						overVBar = true;
						 
						// open the vertical volume bar
						showVolumeBar(true);
					}
					
					// make the over on the button
					overToggle(mcVolumeBt);
					break;	
			}
		}
		
		/**
		 * OUT EVENT HANDLER
		 */
		public function outHandler(event : MouseEvent) : void 
		{
			var mc : MovieClip = event.currentTarget as MovieClip; 
			switch(mc) 
			{
				case mcBack:
				// vertical volume bar
				case mcVolumeBar.mcBack:
				case mcVolumeBar.mcBackBar:
				case mcVolumeBar.mcScrub:
				// volume button
				case mcVolumeBt:
					// hide the voolume bar
					if(verticalVolume)
					{
						if(!volTracking && overVBar) showVolumeBar(false);
						overVBar = false;
					}
					
					// make the over on the sign
					overToggle(mcVolumeBt, false);					
					break;												
			}
		}
		
		public function overToggle(mc : MovieClip, flag : Boolean = true) : void
		{
			if(flag || (!flag && !mc.pressed)) 
			{
				if(mc == mcVolumeBt) 
				{
					mc.mcOver.visible = flag;
					mc.mcSign.setColor(flag ? btSignOverColor : btSignNormalColor);
				}
//				mc.fadeAlpha(flag ? overAlpha : normalAlpha, overTime, "easeOutQuad");
//				if(mc.mcBack && mc.mcBack.mcFill) mc.mcBack.mcFill.setColor(flag ? mcPlayer.controlsBackFillOverColor : mcPlayer.controlsBackFillColor, overTime, "easeOutQuad");
//				if(mc.mcBack && mc.mcBack.mcStroke) mc.mcBack.mcStroke.setColor(flag ? mcPlayer.controlsBackStrokeOverColor : mcPlayer.controlsBackStrokeColor, overTime, "easeOutQuad");
			}
		}				
		
		/**
		 * PRESS EVENT HANDLER
		 */
		public function pressHandler(event : MouseEvent) : void 
		{
			var mc : MovieClip = MovieClip(event.currentTarget);
			
			// set pressed the new pressed object
			currentPressedObject = mc;
			//trace("PRESS:", currentPressedObject.name);
			
			// change the status
			mc.pressed = true;
			
			// make the action
			switch(currentPressedObject) 
			{
				// if is the vertical volume bar - start the seeking interval
				case mcVolumeBar.mcScrub:
				case mcVolumeBar.mcBackBar:
					// change the volume status
					volTracking = true;
					mcVolumeBt.pressed = true;
					
					// retain the volume that was before seeking
					if(getVolume != 0) mcVolumeBt.volume = getVolume;
					
					// start seeking the bar
					seekVolBar();
					clearInterval(intervalId);
					intervalId = setInterval(seekVolBar, 10);
					break;
			}	
			
			// add the release outside event handler
			stage.addEventListener(MouseEvent.MOUSE_UP, releaseOutsideHandler);
			stage.addEventListener(Event.MOUSE_LEAVE, releaseOutsideHandler);								
		}

		/**
		 * RELEASE OUTSIDE EVENT HANDLER
		 */
		public function releaseOutsideHandler(event : Event) : void 
		{
			// remove the release outside event handler
			stage.removeEventListener(MouseEvent.MOUSE_UP, releaseOutsideHandler);
			stage.removeEventListener(Event.MOUSE_LEAVE, releaseOutsideHandler);
	        
	        // here add the code for items that need releaseoutside code
	        if(currentPressedObject != null)
	        {
	        	// change status
	        	currentPressedObject.pressed = false;
	        	
	        	//trace("RELEASEOUTSIDE:",currentPressedObject.name);
				switch(currentPressedObject) 
				{
					// for volume control
					case mcVolumeBar.mcBackBar:
					case mcVolumeBar.mcScrub:
						releaseFunction(currentPressedObject);
						overToggle(mcVolumeBt, false);
						break;
					case mcVolumeBt:	
						overToggle(mcVolumeBt, false);
						break;						
				}
	        }
	        currentPressedObject = null;
		} 

		/**
		 * RELEASE EVENT HANDLER
		 */
		public function releaseHandler(event : MouseEvent) : void 
		{
			var mc : MovieClip = event.currentTarget as MovieClip;
			
			// test if the releasse event comes from the same item that was pressed
			if(mc == currentPressedObject) releaseFunction(mc);
		}
		// release function
		public function releaseFunction(mc : MovieClip) : void 
		{
			//trace("RELEASE:", mc);
			
			// change the status
			mc.pressed = false;
			
			
			// make the action
			switch(mc) 
			{
				// FOR VOLUME CONTROLS
				case mcVolumeBar.mcBackBar:
				case mcVolumeBar.mcScrub:
					// stop adjusting the volume seek bar
					clearInterval(intervalId);
	        		
					// change the status
					volTracking = false;
					mcVolumeBt.pressed = false;
	        		
					// close the bar if is the case
					if(verticalVolume && !overVBar) showVolumeBar(false);
	        		
					break;	 
					           	
				case mcVolumeBt:
					if(mcVolumeBt.mcOff.alpha == 0) // if was loud make it mute
					{
						// retain volume on the bar before seeking
						mcVolumeBt.volume = getVolume;
						
						// set the volume to mute
						setVolume = 0;
					}
					else // if was mute make it loud
					{
						// set the volume that was before mute
						setVolume = mcVolumeBt.volume;
					}
					break;
					
			}	
			
			// change the pressed object to null
			currentPressedObject = null;		
		}

		/*****************************************************
		 * 
		 * 					VOLUME FUNCTIONS
		 * 					
		 *****************************************************/
		// shows the vertical volume bar on over / out the volume bt 
		private function showVolumeBar(flag : Boolean) : void
		{
			var goToY : Number = mcMask.y + 2 * mcMask.height;
			if(flag) goToY -= mcMask.height;
			
			// move the volume bar
			Tweener.addTween(mcVolumeBar, { y : goToY, time: overTime / 2, transition:"easeOutQuad"});
		}		 
		 
		/**
		 * sets the video volume level on the volume vertical bar and the video volume 
		 * called with interval when the user drags on the volume bar
		 */
		private function seekVolBar() : void
		{
			var volMouseX : Number = mcVolumeBar.mouseX -  mcVolumeBar.mcBackBar.x; //trace("volMouseX:", volMouseX);
			var boundLeft : Number = mcVolumeBar.mcScrub.width / 2;
			var boundRight : Number = mcVolumeBar.mcBackBar.width - boundLeft;
			volMouseX = (volMouseX < boundLeft ? boundLeft : (volMouseX > boundRight ? boundRight : volMouseX));
			
			var currentVolume : Number = (volMouseX - boundLeft) / (boundRight - boundLeft);
			//trace("currentVolume:", currentVolume);		
			
			// set the volume
			setVolume = currentVolume;
		}

		/**
		 * changes the volume level to the parameter procent of bar
		 */
		public function set setVolume(volProc : Number) : void 
		{
			// CHECK THE PERCENT
			volProc = (volProc < 0 ? 0 : (volProc > 1 ? 1 : volProc));
			
			// set level on bar
			mcVolumeBar.mcSeekBar.width = (mcVolumeBar.mcScrub.width / 2) + volProc * (mcVolumeBar.mcBackBar.width - mcVolumeBar.mcScrub.width);
			mcVolumeBar.mcScrub.x = mcVolumeBar.mcSeekBar.x + mcVolumeBar.mcSeekBar.width;
			//trace("currentVol:", getVolume);
			
			// CHANGE THE VOLUME SIGN
			setVolumeSign(volProc);		
			
			// ADJUST THE EXTERNAL SOUND VOLUME
			if(extSetSoundFunction != null && extSetSoundFunction.length) extSetSoundFunction(volProc);
		}

		/**
		 * Returns the current volume level from the volume bar
		 */
		public function get getVolume() : Number 
		{
			var volume : Number = (mcVolumeBar.mcSeekBar.width - mcVolumeBar.mcScrub.width / 2) / (mcVolumeBar.mcBackBar.width - mcVolumeBar.mcScrub.width);
			return volume;
		}	
		
		/**
	     * Sets the audio's volume
	     * @param pVolume the new volume's value
	     */
	    public function set setSoundVolume(pVolume : Number) : void 
	    {
	    	Tweener.removeTweens(st);
	        st.volume = pVolume;
			sc.soundTransform = st;
	    }
	    public function set fadeSoundVolume(pVolume : Number) : void 
	    {
			Tweener.addTween(st, { volume:pVolume, time: fadeSoundTime, transition:"easeOutQuad", onUpdate : function() : void 
			{
				if(sc != null) sc.soundTransform = st;
			}});
	    }

		/*
		 * changes the volume sign on pressing the button or on 0 volume
		 */
		private function setVolumeSign(volume : Number) : void
		{
			// ALPHA and COLOR
			if(volume <= 0)
			{
				mcVolumeBt.mcOff.alpha = 1;
				//mcVolumeCt.mcVolumeBt.mcSign.setColor(muteVolColor);
			}
			else
			{
				mcVolumeBt.mcOff.alpha = 0;
				//mcVolumeCt.mcVolumeBt.mcSign.setColor(normalVolColor);
			}
			
			
			// LINES on frames
			var visible : Number = 0;
			if(volume > 0)
			{
				visible = Math.floor(volume * 4);
				if(visible == 4) visible--; 
				visible += 2; //trace(visible+1);
			}
			else
			{
				visible = 1;
			}
			mcVolumeBt.mcSign.gotoAndStop(visible);
			
			
			
			// LINES visible
			/*
			// alpha of the sign
			if(volume <= 0)
				mcAudioCt.mcVolumeBt.mcSign.alpha = 0.5;
			else
				mcAudioCt.mcVolumeBt.mcSign.alpha = 1;
			
			// visible lines
			if(volume == 0) 
			{
				for(var i : Number = 1; i <= 4; i++)
					mcAudioCt.mcVolumeBt.mcSign["mcLine" + i].visible = false;
			}
			else
			{
				visible = Math.ceil(volume * 4); //trace(visible);
				for(i = 1; i <= visible; i++)
					mcAudioCt.mcVolumeBt.mcSign["mcLine" + i].visible = true;
				for(i = visible + 1; i <= 4; i++)
					mcAudioCt.mcVolumeBt.mcSign["mcLine" + i].visible = false;
			}
			*/
		}	

		
		
	} // FROM CLASS
}// FROM PACKAGE