﻿/**
* Tooltip Class - June 2010
* @author Andrei Neamtu
* @version 1; 
* http://www.vuzum.com
* email: office@vuzum.com
*/
 
package com.vuzum.utils
{
	// FLASH IMPORTS
	import caurina.transitions.Tweener;
	import flash.utils.setTimeout;
	import flash.utils.clearTimeout;
	import flash.filters.GlowFilter;
	import flash.events.Event;
	import flash.display.MovieClip;

	// THE CLASS
	public class Rotator extends MovieClip 
	{
		/**
		 * COMPONENTS INSIDE
		 */
		public var mcMask : MovieClip;
		public var mcLabel : MovieClip;
		public var mcText : MovieClip;
		public var mcAllItems : MovieClip;
		public var mcBack : MovieClip;
		
		/**
		 * COMPONENT'S PARAMS
		 */
		// component's dimensions
		private var componentWidth : Number;
		private var componentHeight : Number;		 
		// path to files
		private var pathToFiles : String = "autoplayimagegallery/";
		// XML path
		private var xmlPath : String = "xml/gallery.xml";
		//
		private var direction : String = "up";
		public var labelsColor : Number;
		public var textsColor : Number;
		
		// check function
		private var checkFunction : Function;
		
		// colors
		private var strokeColor : Number;
		private var fillColor : Number;
		// text
		private var textColor : Number;
		private var textSize : Number = 12;
		public var floatTextStep : Number = 1;
		// rotate settings
		public var movementDelay : Number = 2;
		
		// AUX VARS
		private var movementCurrPos : Number;
		private var movementID : uint;
		
		public var textRightToLeft : Boolean;
		

		/**
		 * CONSTRUCTOR
		 */
		public function Rotator() 
		{
			mcAllItems.mcItem.visible = false;
			mcAllItems.mcItem.scaleX = mcAllItems.mcItem.scaleY = 0;
		}
		
		/**
		 * INIT THE COMPONENT
		 */
		public function initWithXMLList(pWidth : Number = NaN, pHeight : Number = NaN, xmlObject : * = null) : void
		{
			// read COMPONENT DIMENSIONS from XML if they were not defined in flash vars
			if(isNaN(componentWidth)) componentWidth = Prototypes.readNumberVar(xmlObject.@width, stage.stageWidth);
			if(isNaN(componentHeight)) componentHeight = Prototypes.readNumberVar(xmlObject.@height, stage.stageHeight);
						
			// read the settings
			strokeColor = Prototypes.readNumberVar(xmlObject.strokeColor); //trace("strokeColor:", strokeColor);
			fillColor = Prototypes.readNumberVar(xmlObject.fillColor); //trace("fillColor:", fillColor);
			textColor = Prototypes.readNumberVar(xmlObject.textColor);
			textSize = Prototypes.readNumberVar(xmlObject.textSize, textSize);
			
			// call the init function			
			init(pWidth, pHeight);
		}
		
		public function init(pWidth : Number = NaN, pHeight : Number = NaN, pPathToFiles : String = "", pXmlPath : String = "") : void
		{
			// DIMENSIONS
			componentWidth = pWidth;
			componentHeight = pHeight;
			
			// PATH TO FILES
			pathToFiles = Prototypes.readPathTo(stage.loaderInfo.parameters.pathToFiles, pPathToFiles);
			
			// XML PATH
			xmlPath = Prototypes.readStringVar(stage.loaderInfo.parameters.xmlPath, pXmlPath);
		}
		
		public function initWithArray(pWidth : Number = NaN, pHeight : Number = NaN, pArray : Array = null, pDirection : String = "up") : void
		{
			// background
			mcBack.width = pWidth;
			mcBack.height = pHeight;

			// direction
			direction = Prototypes.readStringVar(pDirection, "up");
			
			// list of items
			createList(pArray);
		}		
		
		public function clearList() : void
		{
			// stop movement
			toggleMovement(false);
			movementCurrPos = mcAllItems.x = mcAllItems.y = 0;
			
			// clear items
			if(mcAllItems.length > 0)
				for(var i : Number = 0; i < mcAllItems.length; i++)
					if(mcAllItems.contains(mcAllItems["mcItem" + i])) mcAllItems.removeChild(mcAllItems["mcItem" + i]);
		}

		public function createList(pArray : Array = null) : void
		{
			// clear current list
			clearList();
			
			// create the new list
			var mcItem : *;
			mcAllItems.length = (pArray ? pArray.length : 0);
			if(mcAllItems.length > 0)
			{
				for(var i : Number = 0; i < mcAllItems.length; i++)
				{
					// create the item
					mcItem = mcAllItems.duplicateMovieClip(new mcRotatorItem(), "mcItem" + i);
					
					// populate the content
					mcItem.mcLabel.txt.initText(pArray[i].label); //trace(pArray[i].label);
					mcItem.mcLabel.setColor(labelsColor);
					mcItem.mcLabel.mcMask.width = Math.min(mcItem.mcLabel.txt.width, mcBack.width / 2);
					//
					mcItem.mcText.txt.initText(pArray[i].text); //trace(pArray[i].text);
					mcItem.mcText.setColor(textsColor);
					mcItem.mcText.x = mcItem.mcLabel.mcMask.width;
					mcItem.mcText.mcMask.width = Math.min(mcItem.mcText.txt.width, mcBack.width - mcItem.mcText.x);
					
					if(textRightToLeft)
					{
						mcItem.mcText.x = 0;
						mcItem.mcLabel.x = mcItem.mcText.width;
					}
					
					// position
					if(direction == "up" || direction == "down") mcItem.y = (i == 0 ? 0 : mcAllItems["mcItem" + (i-1)].y + mcAllItems["mcItem" + (i-1)].height);
					else mcItem.x = (i == 0 ? 0 : mcAllItems["mcItem" + (i-1)].x + mcAllItems["mcItem" + (i-1)].width);
				}
				
				// check float on the first item
				mcAllItems["mcItem" + 0].mcText.checkFloat(floatTextStep);
					
				// start movement
				toggleMovement();
			}			
		}
		
		public function toggleMovement(flag : Boolean = true) : void
		{
			clearTimeout(movementID);
			
			if(!flag) Tweener.removeTweens(mcAllItems);
			else callDelayMove();
		}
		
		public function moveItems() : void
		{
			if(mcAllItems.length)
			{
				movementCurrPos = movementCurrPos - mcAllItems["mcItem" + 0].height;
				
				Tweener.addTween(mcAllItems, {y : movementCurrPos, time : 0.5, transition : "easeOutExpo", onComplete : function() : void
				{
					// move the items above the mask
					var mcItem : MovieClip;
					if(mcAllItems.length > 0)
						for(var i : Number = 0; i < mcAllItems.length; i++)
						{
							// create the item
							mcItem = mcAllItems["mcItem" + i];
							
							// move it under the mask
							if(mcAllItems.y + mcItem.y < mcMask.y) 
							{
								mcItem.y = mcAllItems.height;
								mcItem.mcText.stopFloat();
							}
							
							// start floating if is on the mask
							if(mcAllItems.y + mcItem.y == mcMask.y) 
							{
								mcItem.mcText.checkFloat(floatTextStep);
//								trace("start float on item:", mcItem.name);
							}
					}
					
					// call after a time the move function
					callDelayMove();				
				}});
			} 
		}
		
		public function callDelayMove() : void
		{
			clearTimeout(movementID);
			movementID = setTimeout(moveItems, movementDelay * 1000);				
		}		

		/*
		 * RESIZE FUNCTION
		 */
		public function resize(pWidth : Number = NaN, pHeight : Number = NaN) : void
		{
			mcMask.width = pWidth;
			mcMask.height = pHeight;
		}
		
		/*
		 * write function
		 */
		public function writeTooltip(str : String = "") : void
		{
			// write & format the text
			mcText.txt.initText(str);
			mcText.txt.format(textSize, textColor);			
			//
			mcBack.mcStroke.mcBody.width = 2 * mcText.x + mcText.txt.width;
			mcBack.mcStroke.mcBody.height = 2 * mcText.y + mcText.txt.height;
			mcBack.mcStroke.mcArrow.centerOn(mcBack.mcStroke.mcBody.width);
			mcBack.mcStroke.mcArrow.y = mcBack.mcStroke.mcBody.height;
			//
			mcBack.mcFill.mcBody.width = mcBack.mcStroke.mcBody.width - 2 * mcBack.mcFill.x;
			mcBack.mcFill.mcBody.height = mcBack.mcStroke.mcBody.height - 2 * mcBack.mcFill.y;
			mcBack.mcFill.mcArrow.centerOn(mcBack.mcFill.mcBody.width);
			mcBack.mcFill.mcArrow.y = mcBack.mcFill.mcBody.height; 
		}
		
		/*
		 * TOGGLE POSITIONING TH TOOLTIP
		 */
		public function togglePosition(pTarget : MovieClip, flag : Boolean = true) : void
		{
			// start enterframe
			if(flag) 
			{
				if(mcText.txt.text != "") pTarget.addEventListener(Event.ENTER_FRAME, itemOverTooltipEnterFrame);
			}
			else pTarget.removeEventListener(Event.ENTER_FRAME, itemOverTooltipEnterFrame);
			
			// show / hide
			this.visible = (!flag || (flag && mcText.txt.text != "") ? flag : false);			
		}
		private function itemOverTooltipEnterFrame(event : Event = null) : void
		{
			// position the arrow
			mcBack.mcStroke.mcArrow.x = Math.round(event.target.mouseX - this.x - mcBack.mcStroke.mcArrow.width / 2);
//			if(mcBack.mcStroke.mcArrow.x < tooltipArrowMargin) mcBack.mcStroke.mcArrow.x = tooltipArrowMargin;
//			if(mcBack.mcStroke.mcArrow.x + mcBack.mcStroke.mcArrow.width > mcBack.mcStroke.mcBody.width - tooltipArrowMargin) mcBack.mcStroke.mcArrow.x = - mcBack.mcStroke.mcArrow.width + mcBack.mcStroke.mcBody.width - tooltipArrowMargin; 
			//
			mcBack.mcFill.mcArrow.x = mcBack.mcStroke.mcArrow.x + mcBack.mcFill.x;						
			
			// move the tooltip after the mouse position
			this.x = Math.round(event.target.mouseX - mcBack.width / 2);
			this.y = Math.round(event.target.mouseY - mcBack.height);
			
			// call the check function
			if(checkFunction != null) checkFunction();
		}				
		
		public function set setCheckFunction(pCheckFunction : Function) : void
		{
			// set the check function
			this.checkFunction = pCheckFunction;
		}	
			
	} // FROM CLASS
} // FROM PACKAGE