﻿/********************************************************
 * Scroller class (AS3)
 * Copyright (c) 2010 Vuzum LLC <http://www.vuzum.com/>
 ********************************************************/
package com.vuzum.utils 
{
	// FLASH IMPORTS
	import flash.utils.clearInterval;
	import flash.utils.setInterval;
	import flash.display.*;
	import flash.events.*;
	
	// OTHER IMPORTS
	import caurina.transitions.*;	
	
	// VUZUM IMPORTS	

	// THE CLASS
	public class Scroller extends MovieClip
	{
		// COMPONENTS INSIDE
		public var mcTrack : MovieClip;
		public var mcScrub : MovieClip;
		public var mcLeftBt : MovieClip;
		public var mcRightBt : MovieClip;
		public var mcBack : MovieClip;
		
		/*
		 * INTERNAL MEMBERS
		 */
		// COMPONENT VARS
		public var componentWidth : Number;
		public var componentHeight : Number;
		
		public var pathToFiles : String = "";
		public var xmlPath : String;
		//		
		private var mcContent : MovieClip;
		private var mcMask : MovieClip;
		
		// DATA VARS
		// handlers vars
		private var currentPressedObject : MovieClip;
		public var interaction : Boolean = false;
		
		// outside funtions
		private var updateFunction : Function; // to be called on moving the content
		public var releaseExtFunction : Function;
		//		
		private var posProperty : String = "x";
		private var posOtherProperty : String = "y";
		//
		private var dimProperty : String = "width";
		private var dimOtherProperty : String = "height";
		//
		private var scaleProperty : String = "scaleX";
		private var scaleOtherProperty : String = "scaleY";
		// float vars		
		private var floatDirection : Boolean = false; // true for left	
		private var isFloating : Boolean = false; // to know if is floating	
		private var floatStep : Number = 1;	// pixels
		//
		public var xMax : Number;
		public var yMax : Number;
		//
		public var xMin : Number;
		public var yMin : Number;		
		
		// SETTINGS VARS
		// type
		private var isVertical : Boolean = false;
		private var updateScrubFlag : Boolean = true;
		private var scrubDefaultDimension : Number;
		// offset
		public var offset : Number = 1;
		public var negOffset : Number;
		public var strokePixelIssue : Number = 0;
		// mouse wheel
		private var mouseWheelFlag : Boolean;
		private var mouseWheelSpeed : Number = 2;
		
		// back and track
		public var backStrokeColor : Number;
		public var backFillColor : Number;
		public var trackFillColor : Number;
		public var trackStrokeColor : Number;
		
		// scrub settings
		public var scrubFillColor : Number;
		public var scrubStrokeColor : Number;
		public var scrubIconColor : Number;
		//
		private var scrubNormalAlpha : Number = 100;
		private var scrubOverAlpha : Number = 60;
		private var scrubOverTweenTime : Number = 0.5;
		//
		public var scrubPercent : Number;
		private var scrubIconMarginAround : Number = 2;
		
		// arrows action settings
		private var controlsPositions : Array = ["sides", "left", "right", "none"];
		private var controlsPosition : String = controlsPositions[0];
		public var controlsDistance : Number = 2;
		//
		public var arrowsBackColor : Number;
		public var arrowsSignColor : Number;
		public var arrowsSignOverColor : Number;
		//
		private var arrowsMoveStep : Number = 10; // in pixels
		private var arrowsMoveStepSpeed : Number = 0;		
		//		
		private var arrowsMoveTimeoutTimeMin : Number = 0.1; // inseconds
		private var arrowsMoveTimeoutTimeMax : Number = 1; // inseconds
		private var arrowsMoveTimeoutTime : Number; // inseconds
		//
		private var arrowsMoveId : uint;
		public var arrowsPercent : Number;

		// AUX VARS
		private var tweenTime : Number = .3;

		/*
		 * CONSTRUCTOR
		 */
		public function Scroller()
		{
			new Prototypes();
			//
			mcScrub.cacheAsBitmap = true;
			scrubPercent = mcScrub.mcSign.height / mcScrub.mcBack.height; //trace(scrubPercent);
			//
			arrowsPercent = mcLeftBt.mcSign.height / mcLeftBt.mcBack.height; //trace(arrowsPercent);
			disable();
		}

		/*
		 * init with an xml object
		 */
		public function initWithXMLList(pWidth : Number = NaN, pHeight : Number = NaN, pMcContent : MovieClip = null, pMcMask : MovieClip = null, xmlObject : * = null) : void
		{
			// read the PARAM DIMENSIONS
			componentWidth = pWidth;
			componentHeight = pHeight;
			
			// read COMPONENT DIMENSIONS from XML if they were not defined in flash vars
			if(isNaN(componentWidth)) componentWidth = Prototypes.readNumberVar(xmlObject.@width, stage.stageWidth);
			if(isNaN(componentHeight)) componentHeight = Prototypes.readNumberVar(xmlObject.@height, stage.stageHeight);
			
			// read the rest of the settings
			if(xmlObject != null)
			{
				// TYPE
				offset = Prototypes.readNumberVar(xmlObject.offset, offset);
				isVertical = Prototypes.readBooleanVar(xmlObject.isVertical, isVertical); //trace("isVertical:", isVertical);
				updateScrubFlag = Prototypes.readBooleanVar(xmlObject.updateScrubToContent, updateScrubFlag); //trace("updateScrubFlag:", updateScrubFlag, xmlObject.updateScrubToContent);
				scrubDefaultDimension = mcScrub[isVertical ? "width" : "height"]; //trace(scrubDefaultDimension);
				scrubDefaultDimension = Prototypes.readNumberVar(xmlObject.updateScrubToContent.@defaultDimension, scrubDefaultDimension); //trace(scrubDefaultDimension);
				
				
				// ARROWS
				controlsPosition = Prototypes.readStringVar(xmlObject.arrows.position, controlsPosition); 
				controlsDistance = Prototypes.readNumberVar(xmlObject.arrows.distanceBetween, controlsDistance);
				// 
				arrowsBackColor = Prototypes.readNumberVar(xmlObject.arrows.backColor, arrowsBackColor); 
				arrowsSignColor = Prototypes.readNumberVar(xmlObject.arrows.signColor, arrowsSignColor); 
				arrowsSignOverColor = Prototypes.readNumberVar(xmlObject.arrows.signOverColor, arrowsSignOverColor); 
				//
				arrowsMoveStep = Prototypes.readNumberVar(xmlObject.arrows.moveStep, arrowsMoveStep);
				
				// MOUSE WHEEL
				mouseWheelFlag = Prototypes.readBooleanVar(xmlObject.mouseWheel, mouseWheelFlag); 
				mouseWheelSpeed = Prototypes.readNumberVar(xmlObject.mouseWheel.@speed, mouseWheelSpeed); 				
				
				// BACK AND TRACK
				backStrokeColor = Prototypes.readNumberVar(xmlObject.backStrokeColor, backStrokeColor); 
				backFillColor = Prototypes.readNumberVar(xmlObject.backFillColor, backFillColor); 
				trackFillColor = Prototypes.readNumberVar(xmlObject.trackFillColor, trackFillColor); 
				trackStrokeColor = Prototypes.readNumberVar(xmlObject.trackStrokeColor, trackStrokeColor); 
				
				// SCRUB COLOR
				scrubFillColor = Prototypes.readNumberVar(xmlObject.scrubFillColor, scrubFillColor); 
				scrubStrokeColor = Prototypes.readNumberVar(xmlObject.scrubStrokeColor, scrubStrokeColor); 
				scrubIconColor = Prototypes.readNumberVar(xmlObject.scrubIconColor, scrubIconColor);
				
				// OVER SETTINGS 
				scrubNormalAlpha = Prototypes.readNumberVar(xmlObject.normalAlpha, scrubNormalAlpha) / 100; 
				scrubOverAlpha = Prototypes.readNumberVar(xmlObject.overAlpha, scrubOverAlpha) / 100;
				scrubOverTweenTime = Prototypes.readNumberVar(xmlObject.controller.controls.overOutTime, scrubOverTweenTime); //trace("scrubOverTweenTime: ", scrubOverTweenTime); 
			}
			
			// init the component with params
			init(componentWidth, componentHeight, 
				 pMcContent, pMcMask, 
				 offset, isVertical, updateScrubFlag, 
				 controlsPosition, controlsDistance, 
				 mouseWheelFlag, mouseWheelSpeed,
				 backFillColor, backStrokeColor, 
				 trackFillColor, trackStrokeColor,
				 scrubFillColor, scrubStrokeColor, scrubIconColor,
				 arrowsBackColor, arrowsSignColor, arrowsSignOverColor,
				 scrubNormalAlpha, scrubOverAlpha);
		}
		
		/*
		 * init with params
		 */
		public function init(pWidth : Number = NaN, pHeight : Number = NaN, 
							 pMcContent : MovieClip = null, pMcMask : MovieClip = null, 
							 pOffset : Number = 0, pIsVertical : Boolean = false, pUpdateScrubDim : Boolean = false, 
							 pControlsPosition : String = "sides", pControlsDistance : Number = 0, 
							 pMouseWheel : Boolean = true, pMouseWheelSpeed : Number = 2, 
							 pBackFillColor : Number = NaN, pBackStrokeColor : Number = NaN, 
							 pTrackFillColor : Number = NaN, pTrackStrokeColor : Number = NaN, 
							 pScrubFillColor : Number = NaN, pScrubStrokeColor : Number = NaN, pScrubIconColor : Number = NaN,
							 pArrowsBackColor : Number = NaN, pArrowsSignColor : Number = NaN, pArrowsSignOverColor : Number = NaN,
							 pNormalAlpha : Number = 1, pOverAlpha : Number = 0.5) : void
		{
			// read the PARAM DIMENSIONS
			componentWidth = pWidth;
			componentHeight = pHeight;
			
			// read the CONTENT and MASK PARAMS			
			mcContent = pMcContent;
			mcMask = pMcMask;
			if(mcContent) mcContent.mask = mcMask;
		
			// read the SETTINGS PARAMS
			negOffset = (pOffset < 0 ? pOffset : NaN);
			offset = (pOffset > 0 ? pOffset : 0);
			//
			mouseWheelFlag = pMouseWheel;
			mouseWheelSpeed = pMouseWheelSpeed;
			//
			updateScrubFlag = pUpdateScrubDim;
			//
			scrubNormalAlpha = pNormalAlpha;
			scrubOverAlpha = pOverAlpha;
			
			
			// show / hide the ARROWS or position them
			if(pControlsPosition == "top") pControlsPosition = "left";
			if(pControlsPosition == "bottom") pControlsPosition = "right";
			mcLeftBt.visible = mcRightBt.visible = !(pControlsPosition == "none");
			controlsPosition = controlsPositions.indexOf(pControlsPosition) >= 0 ? pControlsPosition : "left";
			controlsDistance = pControlsDistance;
			arrowsSignColor = pArrowsSignColor;
			arrowsSignOverColor = pArrowsSignOverColor;

			
			// decide what's the DIRECTION
			isVertical = pIsVertical; //trace("isVertical:", isVertical);
			//
			posProperty = (isVertical ? "y" : "x");
			posOtherProperty = (!isVertical ? "y" : "x");
			//
			dimProperty = (isVertical ? "height" : "width");
			dimOtherProperty = (!isVertical ? "height" : "width");
			//
			scaleProperty = (isVertical ? "scaleY" : "scaleX");
			scaleOtherProperty = (!isVertical ? "scaleY" : "scaleX");
			//

			// ROTATE elements depending on the direction
			if(isVertical)
			{
				// rotate the back's shape
//				mcBack.mcShape.rotation = 90;
//				mcBack.mcShape.x = mcBack.mcShape.width;

				// rotate the track shape
//				mcTrack.mcShape.rotation = 90;
//				mcTrack.mcShape.x = mcBack.mcShape.width;
				
				// rotate the icon inside the scrub
				mcScrub.mcSign.mcShape.rotation = 90;
				mcScrub.mcSign.mcShape.x = mcScrub.mcSign.mcShape.width;
				//
//				mcScrub.mcBack.mcShape.rotation = 90;
//				mcScrub.mcBack.mcShape.x = mcScrub.mcBack.mcShape.width;
            	
				// rotate the left button elements
				mcLeftBt.mcSign.mcShape.rotation = 90;
				mcLeftBt.mcSign.mcShape.x = mcLeftBt.mcSign.mcShape.width;
				//            	
				mcLeftBt.mcBack.mcShape.rotation = 90;
				mcLeftBt.mcBack.mcShape.x = mcLeftBt.mcBack.mcShape.width;
            	
				// rotate the right button elements
				mcRightBt.mcSign.mcShape.rotation = 90;
				mcRightBt.mcSign.mcShape.x = mcRightBt.mcSign.mcShape.width;
				//            	
				mcRightBt.mcBack.mcShape.rotation = 90;
				mcRightBt.mcBack.mcShape.x = mcRightBt.mcBack.mcShape.width;            	            	 
			}
			mcLeftBt.mcBack.alpha = scrubNormalAlpha;
			mcRightBt.mcBack.alpha = scrubNormalAlpha;
            

			// set the COLORS
			setColors(pBackFillColor, pBackStrokeColor, pTrackFillColor, pTrackStrokeColor, pScrubFillColor, pScrubStrokeColor, pScrubIconColor, pArrowsBackColor, pArrowsSignColor);            
            
            
			// init SCRUB		
			mcScrub.mouseChildren = false;
			mcScrub.mcBack.alpha = scrubNormalAlpha; //trace(mcScrub.mcBack.alpha);


			// init TRACKPAD
//			mcTrack.buttonMode = false;
			
			
			// MOUSE WHEEL EVENT HANDLER
			if(mouseWheelFlag) stage.addEventListener(MouseEvent.MOUSE_WHEEL, onMouseWheel);
			
			
			// RESIZE scroll bar if is the case
			if(isVertical) resize(pWidth, (!isNaN(pHeight) ? pHeight : (mcMask != null ? mcMask.height : NaN)));
			else resize((!isNaN(pWidth) ? pWidth : (mcMask != null ? mcMask.width : NaN)), pHeight);
		}
		/*
		 * change the colors
		 */
		public function setColors(pBackFillColor : Number = NaN, pBackStrokeColor : Number = NaN, 
								  pTrackFillColor : Number = NaN, pTrackStrokeColor : Number = NaN, 
								  pScrubFillColor : Number = NaN, pScrubStrokeColor : Number = NaN, pScrubIconColor : Number = NaN,
								  pScrollControlsBackColor : Number = NaN, pScrollControlsIconsColor : Number = NaN) : void
		{
			// BACK
			mcBack.mcFill.setColor(pBackFillColor);
			if(pBackStrokeColor < 0) mcBack.mcStroke.visible = false;
			else mcBack.mcStroke.setColor(pBackStrokeColor);
			
			// TRACK
			mcTrack.mcFill.setColor(pTrackFillColor);
			//trace("pTrackStrokeColor:", pTrackStrokeColor);
			if(pTrackStrokeColor >= 0) mcTrack.mcStroke.setColor(pTrackStrokeColor);
			else mcTrack.mcStroke.visible = false;
			
			// SCRUB
			mcScrub.mcBack.mcFill.setColor(pScrubFillColor);
			//
			if(pScrubStrokeColor < 0) mcScrub.mcBack.mcStroke.visible = false;
			else mcScrub.mcBack.mcStroke.setColor(pScrubStrokeColor);
			//
			if(pScrubIconColor < 0) mcScrub.mcSign.visible = false;
			else mcScrub.mcSign.setColor(pScrubIconColor);
			
			// ARROWS
			mcLeftBt.mcBack.setColor(pScrollControlsBackColor);
			mcLeftBt.mcSign.setColor(pScrollControlsIconsColor);
			//
			mcRightBt.mcBack.setColor(pScrollControlsBackColor);
			mcRightBt.mcSign.setColor(pScrollControlsIconsColor);
		}

		/**
		 * resizes the scroller
		 */
		public function resize(pWidth : Number = NaN, pHeight : Number = NaN) : void
		{
			// resize the BACKGROUND
			mcBack.mcFill.width = mcBack.mcStroke.width = pWidth;
			mcBack.mcFill.height = mcBack.mcStroke.height = pHeight;
			
			// resize LEFT button
			mcLeftBt.mcBack[dimOtherProperty] = mcBack[dimOtherProperty] - 2 * offset;
			mcLeftBt.mcBack[scaleProperty] = mcLeftBt.mcBack[scaleOtherProperty];
			//
//			mcLeftBt.mcSign[dimOtherProperty] = (mcLeftBt.mcSign[dimOtherProperty] > mcLeftBt.mcBack[dimOtherProperty] ? mcLeftBt.mcBack[dimOtherProperty] : mcLeftBt.mcSign[dimOtherProperty]);
			mcLeftBt.mcSign[dimOtherProperty] = Math.round(arrowsPercent * mcLeftBt.mcBack[dimOtherProperty]);
			mcLeftBt.mcSign[scaleProperty] = mcLeftBt.mcSign[scaleOtherProperty];
			mcLeftBt.mcSign.centerOnClip(mcLeftBt.mcBack);
			
			// resize RIGHT button
			mcRightBt.mcBack[dimOtherProperty] = mcBack[dimOtherProperty] - 2 * offset;
			mcRightBt.mcBack[scaleProperty] = mcRightBt.mcBack[scaleOtherProperty];
			//
//			mcRightBt.mcSign[dimOtherProperty] = (mcRightBt.mcSign[dimOtherProperty] > mcRightBt.mcBack[dimOtherProperty] ? mcRightBt.mcBack[dimOtherProperty] : mcRightBt.mcSign[dimOtherProperty]);
			mcRightBt.mcSign[dimOtherProperty] = Math.round(arrowsPercent * mcLeftBt.mcBack[dimOtherProperty]);
			mcRightBt.mcSign[scaleProperty] = mcRightBt.mcSign[scaleOtherProperty];			
			mcRightBt.mcSign.centerOnClip(mcRightBt.mcBack);
			
			// resize and position TRACK
			//trace(dimProperty, mcBack[dimProperty]);
			var trackWidthBeforeResize : Number = mcTrack[dimProperty];
			mcTrack[dimProperty] = mcBack[dimProperty] - (mcLeftBt.visible ? (mcLeftBt.mcBack[dimProperty] + mcRightBt.mcBack[dimProperty] + 2 * controlsDistance) : 0) - 2 * offset;
			mcTrack[dimOtherProperty] = mcBack[dimOtherProperty] - 2 * offset;
			//
			mcTrack[posProperty] = offset + ((!mcLeftBt.visible || controlsPosition == "right") ? 0 : (mcLeftBt.mcBack[dimProperty] + controlsDistance + (controlsPosition == "left" ? mcRightBt.mcBack[dimProperty] + controlsDistance : 0)));
			mcTrack[posOtherProperty] = offset;				
			
			// position LEFT and RIGHT button
			mcLeftBt[posProperty] = offset + (controlsPosition == "right" ? (mcTrack[dimProperty] + controlsDistance) : 0);
			mcLeftBt[posOtherProperty] = offset;						
			//
			mcRightBt[posProperty] = (controlsPosition != "sides") ? (mcLeftBt[posProperty] + mcLeftBt.mcBack[dimProperty] + controlsDistance) : mcTrack[posProperty] + mcTrack[dimProperty] + controlsDistance;
			mcRightBt[posOtherProperty] = offset;		
			

			// resize SCRUB
			// resize the background
			mcScrub.mcBack.mcStroke[dimProperty] = mcScrub.mcBack.mcFill[dimProperty] = scrubDefaultDimension; //trace(dimProperty, mcScrub.mcBack[dimProperty])
			mcScrub.mcBack.mcStroke[dimOtherProperty] = mcScrub.mcBack.mcFill[dimOtherProperty] = mcBack[dimOtherProperty] - 2 * (!isNaN(negOffset) ? negOffset : offset)-strokePixelIssue;
			// resize & centrate the icon in scrub
			mcScrub.mcSign[dimOtherProperty] = Math.round(scrubPercent * mcScrub.mcBack[dimOtherProperty]);
			mcScrub.mcSign[scaleProperty] = mcScrub.mcSign[scaleOtherProperty];
			//mcScrub.mcBack[dimProperty] = mcScrub.mcBack[dimOtherProperty] + 3 / 2 * Math.round((mcScrub.mcBack[dimOtherProperty] - mcScrub.mcSign[dimOtherProperty]));
			mcScrub.mcSign.centerOnClip(mcScrub.mcBack);
			// position the scrub
			mcScrub[posOtherProperty] = Math.round((mcBack[dimOtherProperty] - mcScrub.mcBack[dimOtherProperty]) / 2)-strokePixelIssue;
			mcScrub[posProperty] = (mcScrub[posProperty] * mcTrack[dimProperty]) / trackWidthBeforeResize;
			
			// update MIN and MAX
			this[posProperty + "Min"] = mcTrack[posProperty];
			this[posProperty + "Max"] = mcTrack[posProperty] + mcTrack[dimProperty] - mcScrub[dimProperty];
			
			
			// UPDATE THE SCROLL DIMENSION or disable it
			update();
		}

		/*
		 * updates the scrub width / height depending on the new changes appeared meanwhile on the content
		 */
		public function update() : void
		{
			if(mcContent != null && mcMask != null)
			{
				// update the scrub dimension if is the case					
				if(mcContent[dimProperty] > mcMask[dimProperty] && updateScrubFlag)
				{
					// calculate the new dimension of the scrub
					var tempVar : Number = (mcTrack[dimProperty] * mcMask[dimProperty]) / mcContent[dimProperty];
					if(tempVar < mcScrub.mcSign[dimProperty] + 2 * scrubIconMarginAround) tempVar = mcScrub.mcSign[dimProperty] + 2 * scrubIconMarginAround;
					
					// resize scrub's back and reposition icon inside
					var tweenObject : Object = new Object();
					tweenObject[dimProperty] = tempVar;
					tweenObject.time = tweenTime;
					tweenObject.transition = "easeOutExpo";
					tweenObject.onUpdate = function() : void
					{
						mcScrub.mcSign[posProperty] = Math.round((mcScrub.mcBack[dimProperty] - mcScrub.mcSign[dimProperty]) / 2);
					};
					Tweener.addTween(mcScrub.mcBack.mcFill, tweenObject);					
					Tweener.addTween(mcScrub.mcBack.mcStroke, tweenObject);					
					
					// update the max depending on the new dimension of the scrub
					this[posProperty + "Max"] = mcTrack[posProperty] + mcTrack[dimProperty] - tempVar;
				}
				
				// enable / disable the scrollbar events		
				toggleEnable(mcContent[dimProperty] > mcMask[dimProperty]);
			}
		}

		/**
		 * enable toggle for the scroller
		 */
		public function enable() : void 
		{ 
			toggleEnable(true); 
		}
		public function disable() : void 
		{ 
			toggleEnable(false); 
		}
		public function toggleEnable(flag : Boolean = true) : void
		{
			// change alpha
			this.alpha = (flag ? 1 : 0.3);
						
			// SETUP SCRUB
			mcScrub.visible = flag;
			mcScrub.mcBack.alpha = scrubNormalAlpha;
			// move the scrub
			stopMove();
			checkScrubPos();
			// remove events from SCRUB
			mcScrub.toggleEvents(this, flag);			
			
			// move the content after the scrub
			if(flag) moveContentTo(NaN, 0);
			
			// remove events from TRACK
			mcTrack.toggleEvents(this, flag);
			
			// remove events on sides ARROWS
			mcLeftBt.toggleEvents(this, flag);
			mcRightBt.toggleEvents(this, flag);
			
			// remove MOUSE WHEEL listener
			if(stage)
			{
				stage.removeEventListener(MouseEvent.MOUSE_WHEEL, onMouseWheel);
				if(flag && mouseWheelFlag) stage.addEventListener(MouseEvent.MOUSE_WHEEL, onMouseWheel);
			}
		}		

		/**************************************************
		 * 
		 * 				EVENT HANDLERS
		 * 
		 *************************************************/

		/**
		 * OVER EVENT HANDLER
		 */
		public function overHandler(e : MouseEvent) : void
		{
			overFunction(e.currentTarget as MovieClip);
		}

		/**
		 * OUT EVENT HANDLER
		 */
		public function outHandler(e : MouseEvent) : void
		{
			var mc : MovieClip = e.currentTarget as MovieClip;
			if(mc != currentPressedObject) overFunction(mc, false);
		}

		public function overFunction(mc : MovieClip, flag : Boolean = true) : void
		{
			// arrows
			if(mc.mcSign && mc != mcScrub) mc.mcSign.setColor(flag ? arrowsSignOverColor : arrowsSignColor);
			
			// track and scrub
			if(mc == mcTrack) mc = mcScrub;
			mc.mcBack.fadeAlpha(flag ? scrubOverAlpha : scrubNormalAlpha);
		}				

		/**
		 * PRESS EVENT HANDLER
		 */
		public function pressHandler(e : MouseEvent) : void
		{
			// retain the pressed object
			currentPressedObject = e.currentTarget as MovieClip;
			
			// set the interaction status 
			interaction = true;				
			
			// make the action
			switch(currentPressedObject)
			{
				case mcScrub:
				case mcTrack:
					//currentPressedObject = mcScrub;
				
					// stop the floatin ig is the case
					mcScrub.wasFloating = isFloating;
					if(isFloating) floatScrubToggle(false);
				
					// move the scrub to the mouse position
					//scrubMoveHandler();
				
					// start mouse move listener
					//stage.addEventListener(MouseEvent.MOUSE_MOVE, scrubMoveHandler);
					this.addEventListener(Event.ENTER_FRAME, scrubMoveHandler);
					break;
				
				case mcLeftBt:
				case mcRightBt:
					arrowsMoveTimeoutTime = 0;
					arrowsMoveStepSpeed = 0;
					startArrowMove(true, (currentPressedObject == mcLeftBt));
					break;
			}
			
			// add the release outside event handlers
			stage.addEventListener(MouseEvent.MOUSE_UP, releaseOutsideHandler);
			stage.addEventListener(Event.MOUSE_LEAVE, releaseOutsideHandler);	        
		}

		public function startArrowMove(flag : Boolean = true, direction : Boolean = true) : void
		{
			// stop moving the cursor
			clearInterval(arrowsMoveId);
			
			// start moving the cursor
			if(flag) arrowsMoveId = setInterval(arrowsAction, 100, direction); 
		}

		public function arrowsAction(flag : Boolean = true) : void
		{
			if(arrowsMoveTimeoutTime < arrowsMoveTimeoutTimeMax) 
			{
				arrowsMoveStepSpeed += arrowsMoveStep;
				arrowsMoveTimeoutTime += arrowsMoveTimeoutTimeMin;
			}
			
			// move the scrub to the oposite end of the scroller
			moveScrubTo((mcScrub[posProperty] + (flag ? -1 : 1) * arrowsMoveStepSpeed), arrowsMoveTimeoutTime);
		}
		
		/**
		 * RELEASE OUTSIDE EVENT HANDLER FOR
		 */
		public function releaseOutsideHandler(event : Event) : void 
		{
			// remove the release outside event handlers
			stage.removeEventListener(MouseEvent.MOUSE_UP, releaseOutsideHandler);
			stage.removeEventListener(Event.MOUSE_LEAVE, releaseOutsideHandler);
			
			if(currentPressedObject)
			{
//				mcBack.ttrace(currentPressedObject.name);
				// make action for the current pressed object if needed
				switch(currentPressedObject)
				{
					// call release function
					case mcScrub:
					case mcTrack:
					case mcLeftBt:
					case mcRightBt:
						releaseFunction(currentPressedObject);
						break;
				}
				
				// make out handler
				if(!currentPressedObject.hitTestPoint(stage.mouseX, stage.mouseY)) overFunction(currentPressedObject, false);
			}
		}

		/**
		 * RELEASE EVENT HANDLER
		 */
		public function releaseHandler(e : MouseEvent) : void
		{
			var mc : MovieClip = e.currentTarget as MovieClip;
			if(mc == currentPressedObject) 
			{
				releaseFunction(mc);
				currentPressedObject = null;
			}
		}

		private function releaseFunction(mc : MovieClip) : void
		{
			switch(mc)
			{
				case mcScrub:
				case mcTrack:
					// resume floating if is the case
					if(mcScrub.wasFloating) floatScrubToggle(true);
					
					// stop the event listener				
					//stage.removeEventListener(MouseEvent.MOUSE_MOVE, scrubMoveHandler);
					this.removeEventListener(Event.ENTER_FRAME, scrubMoveHandler);
					break;
					
				case mcLeftBt:				
				case mcRightBt:
					startArrowMove(false, (mc == mcLeftBt));
					arrowsAction(mc == mcLeftBt);
					break;				
			}
			
			// set the interaction status 
			interaction = false;	
		}
		
		/**
		 * MOUSE MOVE EVENT HANDLER
		 */
		//public function scrubMoveHandler(event : MouseEvent = null) : void
		public function scrubMoveHandler(event : Event = null) : void
		{
			// move the srub to the mouse position
			moveScrubTo();
			
			// update event
			//if(event != null) event.updateAfterEvent();
		}

		
		/**
		 * centrates the scrub on the @param position / the mouse position and moves the content after it
		 */
		public function moveScrubTo(pPosition : Number = NaN, pTime : Number = NaN, pOnComletete : Function = null) : void
		{
			// calculate the position
			var goTo : Number = (!isNaN(pPosition) ? pPosition : (this["mouse" + posProperty.toLocaleUpperCase()] - mcScrub.mcBack[dimProperty] / 2));
			if(goTo < this[posProperty + "Min"]) goTo = this[posProperty + "Min"];
			if(goTo > this[posProperty + "Max"]) goTo = this[posProperty + "Max"];
			
			// calculate the time
			pTime = (!isNaN(pTime) ? pTime : tweenTime);
			
			// prepare the movement
			var tweenObject : Object = new Object();
			tweenObject[posProperty] = goTo;
			tweenObject.time = pTime;
			tweenObject.transition = "easeOutExpo";
			tweenObject.onUpdate = function() : void
			{
				// move the content depending on the scrub's new pos
				moveContentTo(NaN, 0);				
			};
			tweenObject.onComplete = function() : void
			{
				//trace(mcScrub[posProperty]);
				if(pOnComletete != null) pOnComletete();
			};
			// move the scrub
			Tweener.addTween(mcScrub, tweenObject);
		}
		/*
		 * to float the scrub to left / right
		 */
		public function floatScrubToggle(startFlag : Boolean = true, step : Number = NaN, pFloatDirection : String = "") : void
		{
			// reset the step if the param is set
			floatStep = (!isNaN(step) ? step : floatStep); //trace("floatStep:", floatStep);
			
			// reset the direction if the param is set
			floatDirection = Prototypes.readBooleanVar(pFloatDirection, floatDirection); //trace("floatDirection:", floatDirection);
			
			// stop the interval
			mcScrub.removeEventListener(Event.ENTER_FRAME, iterateScrubEnterFrame);
			
			// change the status
			isFloating = startFlag;
			
			// restart it if is the case
			if(startFlag) mcScrub.addEventListener(Event.ENTER_FRAME, iterateScrubEnterFrame);			
		}
		/*
		 * called with an interval to float the scrub to left / right
		 */
		public function iterateScrubEnterFrame(event : Event) : void
		{
			iterateScrub(floatStep);
		}
		public function iterateScrub(step : Number) : void
		{
			/******* FLOATING THE CONTENT*******/
			// move the content and the scrub after it
			mcContent[posProperty] += (floatDirection ? -1 : 1) * step;
			updateScrub(NaN, 0);
			
			// change the direction if is the case for the content
			if(mcContent[posProperty] + mcContent[dimProperty] <= mcMask[posProperty] + mcMask[dimProperty]) 
			{
				floatDirection = false;
				//trace("right");
			}
			if(mcContent[posProperty] >= mcMask[posProperty]) 
			{
				floatDirection = true;
				//trace("left");
			}
			
			
			/******* FLOATING THE SCRUB*******/
			// move the scrub agter the content
			//moveScrubTo((mcScrub[posProperty] + (floatDirection ? -1 : 1) * step), 0);			
			
			// change the direction if is the case for the scrub
//			if(mcScrub[posProperty] <= this[posProperty + "Min"]) 
//			{
//				floatDirection = false;
//				//trace("right");
//			}
//			if(mcScrub[posProperty] >= this[posProperty + "Max"]) 
//			{
//				floatDirection = true;
//				//trace("left");
//			}			
		}		
		
		
		/*
		 * moves the scrub depending on the @param or content position
		 * called from outside when the content is moved for example
		 */
		public function updateScrubToPercent(pPercent : Number = NaN, pTweenTime : Number = NaN) : void
		{
			updateScrub(this[posProperty + "Min"] + pPercent * (this[posProperty + "Max"] - this[posProperty + "Min"]), (!isNaN(pTweenTime) ? pTweenTime : tweenTime));
		}
		public function updateScrub(pPosition : Number = NaN, pTweenTime : Number = NaN) : void
		{
			// calculate the content position
			if(mcContent)
			{
				pPosition = (!isNaN(pPosition) ? pPosition : mcContent[posProperty]);
				pPosition = (mcContent ? this[posProperty + "Max"] * ( (mcMask[posProperty] - pPosition) / (mcContent[dimProperty] - mcMask[dimProperty])) : pPosition);
				if(pPosition > this[posProperty + "Max"]) pPosition = this[posProperty + "Max"];
				if(pPosition < this[posProperty + "Min"]) pPosition = this[posProperty + "Min"];
	            
				// move the scrub depending the content position
				var tweenObject : Object = new Object();
				tweenObject[posProperty] = pPosition;
				tweenObject.time = (!isNaN(pTweenTime) ? pTweenTime : tweenTime); //trace(tweenObject.time, pTweenTime);
				tweenObject.transition = "easeOutExpo";
				Tweener.addTween(mcScrub, tweenObject);
			}
		}
		
		/*
		 * checks the position of the scrub for not passing the min and max positions
		 */		
		public function checkScrubPos() : void
		{
			if(mcScrub[posProperty] < this[posProperty + "Min"]) mcScrub[posProperty] = this[posProperty + "Min"];
			if(mcScrub[posProperty] > this[posProperty + "Max"]) mcScrub[posProperty] = this[posProperty + "Max"];			
		}				
				
		/*
		 * stop all movement
		 */
		public function stopMove() : void
		{
//			stage.removeEventListener(MouseEvent.MOUSE_MOVE, scrubMoveHandler);
//			stage.removeEventListener(MouseEvent.MOUSE_WHEEL, onMouseWheel);
			Tweener.removeTweens(mcContent);
			Tweener.removeTweens(mcScrub);
		}		
		

		/**
		 * MOUSE WHEEL EVENT HANDLER
		 */
		public function onMouseWheel(e : MouseEvent) : void
		{
			var delta : Number = e.delta; //mcBack.ttrace(delta);
	    	
			if(((mcContent != null && mcContent.hitTestPoint(stage.mouseX, stage.mouseY)) || 
				(mcMask != null && mcMask.hitTestPoint(stage.mouseX, stage.mouseY)) || 
				this.hitTestPoint(stage.mouseX, stage.mouseY)) && this.parent.visible)
			{
				mcScrub[posProperty] -= mouseWheelSpeed * delta;	    		
				checkScrubPos();
				
				// move the content after the new position of the scrub
				moveContentTo(NaN, 0);
			}
		}				

		/**
		 * moves the content depending on the scrub's position
		 */
		public function moveContentToPercent(pProcent : Number = NaN, pTime : Number = NaN) : void
		{
			// calculate the percent
			pProcent = (!isNaN(pProcent) ? pProcent : (mcScrub[posProperty] - this[posProperty + "Min"]) / (this[posProperty + "Max"] - this[posProperty + "Min"]));
			
			// calculate the position
			var goTo : Number = mcMask[posProperty] - pProcent * (mcContent[dimProperty] - mcMask[dimProperty]);
			
			// move the content
			moveContentTo(goTo, pTime);
		}
		public function moveContentTo(goTo : Number = NaN, pTime : Number = NaN) : void
		{
			// calculate the position
			goTo = (!isNaN(goTo) ? goTo : mcMask[posProperty] - ((mcScrub[posProperty] - this[posProperty + "Min"]) / (this[posProperty + "Max"] - this[posProperty + "Min"])) * (mcContent[dimProperty] - mcMask[dimProperty]));
			
			// calculate the time
			pTime = (!isNaN(pTime) ? pTime : tweenTime);
			
			// move the content
			if(mcContent)
			{
				// check the content's new position
				if(goTo + mcContent[dimProperty] < mcMask[posProperty] + mcMask[dimProperty]) goTo = mcMask[posProperty] + mcMask[dimProperty] - mcContent[dimProperty];
				if(goTo > mcMask[posProperty]) goTo = mcMask[posProperty];	
				
				// move the content		
				var tweenObject : Object = new Object();
				tweenObject[posProperty] = goTo;
				tweenObject.time = pTime;
				tweenObject.transition = "easeOutExpo";            
				tweenObject.onUpdate = function() : void 
				{
					if(updateFunction != null) updateFunction();
				};
				// call the external release function after the tween is done
				tweenObject.onComplete = function() : void 
				{
					// call the the release external function
					if(releaseExtFunction != null) releaseExtFunction();
				};
				Tweener.addTween(mcContent, tweenObject);
			}		
		}		


		/**************************************************
		 * 
		 * 				AUX FUNCTIONS
		 * 
		 *************************************************/	
		public function set setUpdateFunction(value : Function) : void
		{
			updateFunction = value;
		}

		public function set setReleaseFunction(value : Function) : void
		{
			releaseExtFunction = value;
		}		
		
	}// FROM CALSS
}// FROM PACKAGE
