﻿/**
* Tooltip Class - June 2010
* @author Andrei Neamtu
* @version 1; 
* http://www.vuzum.com
* email: office@vuzum.com
*/
 
package com.vuzum.utils
{
	// FLASH IMPORTS
	import flash.filters.GlowFilter;
	import flash.events.Event;
	import flash.display.MovieClip;

	// THE CLASS
	public class Tooltip extends MovieClip 
	{
		/**
		 * COMPONENTS INSIDE
		 */
		public var mcText : MovieClip;
		public var mcBack : MovieClip;
		
		/**
		 * COMPONENT'S PARAMS
		 */
		// component's dimensions
		private var componentWidth : Number;
		private var componentHeight : Number;		 
		// path to files
		private var pathToFiles : String = "autoplayimagegallery/";
		// XML path
		private var xmlPath : String = "xml/gallery.xml";
		
		// check function
		private var checkFunction : Function;
		private var strokeColor : Number;
		private var fillColor : Number;
		private var textColor : Number;
		private var textSize : Number = 12;
		
		// DATA VARS
		private var tooltipArrowMargin : Number = 10;
		

		/**
		 * CONSTRUCTOR
		 */
		public function Tooltip() 
		{
			this.mouseEnabled = this.mouseChildren = false;
		}
		
		/**
		 * INIT THE COMPONENT
		 */
		public function initWithXMLList(pWidth : Number = NaN, pHeight : Number = NaN, xmlObject : * = null) : void
		{
			// read COMPONENT DIMENSIONS from XML if they were not defined in flash vars
			if(isNaN(componentWidth)) componentWidth = Prototypes.readNumberVar(xmlObject.@width, stage.stageWidth);
			if(isNaN(componentHeight)) componentHeight = Prototypes.readNumberVar(xmlObject.@height, stage.stageHeight);
						
			// read the settings
			strokeColor = Prototypes.readNumberVar(xmlObject.strokeColor); //trace("strokeColor:", strokeColor);
			fillColor = Prototypes.readNumberVar(xmlObject.fillColor); //trace("fillColor:", fillColor);
			textColor = Prototypes.readNumberVar(xmlObject.textColor);
			textSize = Prototypes.readNumberVar(xmlObject.textSize, textSize);
			
			// call the init function			
			init(pWidth, pHeight);
		}
		
		public function init(pWidth : Number = NaN, pHeight : Number = NaN, pPathToFiles : String = "", pXmlPath : String = "") : void
		{
			// DIMENSIONS
			componentWidth = pWidth;
			componentHeight = pHeight;
			
			// PATH TO FILES
			pathToFiles = Prototypes.readPathTo(stage.loaderInfo.parameters.pathToFiles, pPathToFiles);
			
			// XML PATH
			xmlPath = Prototypes.readStringVar(stage.loaderInfo.parameters.xmlPath, pXmlPath);
			
			// color the background
			mcBack.mcStroke.setColor(strokeColor);
			mcBack.mcFill.setColor(fillColor);
			mcBack.filters = [new GlowFilter(strokeColor, 1, 5, 5, 2, 1)];
			
			// format the text
			mcText.txt.format(textSize, textColor);
		}
		
		public function resize(pWidth : Number = NaN, pHeight : Number = NaN) : void
		{
			
		}
		
		/*
		 * write function
		 */
		public function writeTooltip(str : String = "") : void
		{
			// write & format the text
			mcText.txt.initText(str);
			mcText.txt.format(textSize, textColor);			
			//
			mcBack.mcStroke.mcBody.width = 2 * mcText.x + mcText.txt.width;
			mcBack.mcStroke.mcBody.height = 2 * mcText.y + mcText.txt.height;
			mcBack.mcStroke.mcArrow.centerOn(mcBack.mcStroke.mcBody.width);
			mcBack.mcStroke.mcArrow.y = mcBack.mcStroke.mcBody.height;
			//
			mcBack.mcFill.mcBody.width = mcBack.mcStroke.mcBody.width - 2 * mcBack.mcFill.x;
			mcBack.mcFill.mcBody.height = mcBack.mcStroke.mcBody.height - 2 * mcBack.mcFill.y;
			mcBack.mcFill.mcArrow.centerOn(mcBack.mcFill.mcBody.width);
			mcBack.mcFill.mcArrow.y = mcBack.mcFill.mcBody.height; 
		}
		
		/*
		 * TOGGLE POSITIONING TH TOOLTIP
		 */
		public function togglePosition(pTarget : MovieClip, flag : Boolean = true) : void
		{
			// start enterframe
			if(flag) 
			{
				if(mcText.txt.text != "") pTarget.addEventListener(Event.ENTER_FRAME, itemOverTooltipEnterFrame);
			}
			else pTarget.removeEventListener(Event.ENTER_FRAME, itemOverTooltipEnterFrame);
			
			// show / hide
			this.visible = (!flag || (flag && mcText.txt.text != "") ? flag : false);			
		}
		private function itemOverTooltipEnterFrame(event : Event = null) : void
		{
			// position the arrow inside
//			mcBack.mcStroke.mcArrow.x = Math.round(event.target.mouseX - this.x - mcBack.mcStroke.mcArrow.width / 2);
			mcBack.mcStroke.mcArrow.x = Math.round((mcBack.mcStroke.width - mcBack.mcStroke.mcArrow.width) / 2);
			//
			if(mcBack.mcStroke.mcArrow.x < tooltipArrowMargin) mcBack.mcStroke.mcArrow.x = tooltipArrowMargin;
			if(mcBack.mcStroke.mcArrow.x + mcBack.mcStroke.mcArrow.width > mcBack.mcStroke.mcBody.width - tooltipArrowMargin) mcBack.mcStroke.mcArrow.x = - mcBack.mcStroke.mcArrow.width + mcBack.mcStroke.mcBody.width - tooltipArrowMargin; 
			//
			mcBack.mcFill.mcArrow.x = mcBack.mcStroke.mcArrow.x + mcBack.mcFill.x;						
			
			// move the tooltip after the mouse position
			this.x = Math.round(event.target.mouseX - mcBack.width / 2);
			this.y = Math.round(event.target.mouseY - mcBack.height);
			
			// call the check function
			if(checkFunction != null) checkFunction();
		}				
		
		public function set setCheckFunction(pCheckFunction : Function) : void
		{
			// set the check function
			this.checkFunction = pCheckFunction;
		}	
			
	} // FROM CLASS
} // FROM PACKAGE